@@ -28,16 +28,10 @@
 #define __MOS_OS_H__
 
 #include "mos_defs.h"
-#include "media_skuwa_specific.h"
 #include "mos_utilities.h"
-#include "mos_utilities_next.h"
-#include "mos_util_debug_next.h"
+#include "media_skuwa_specific.h"
+#include "mos_util_debug.h"
 #include "mos_os_hw.h"         //!< HW specific details that flow through OS pathes
-#ifndef MEDIA_SOFTLET
-#include "mos_os_cp_interface_specific.h"         //!< CP specific OS functionality
-#else
-class MosCpInterface;
-#endif
 #if (_RELEASE_INTERNAL || _DEBUG)
 #if defined(CM_DIRECT_GUC_SUPPORT)
 #include "work_queue_mngr.h"
@@ -223,25 +217,6 @@ private:
 #define E_UNEXPECTED    0x8000FFFFL
 #endif // E_UNEXPECTED
 
-//!
-//! \brief Enum for OS component
-//!
-enum MOS_COMPONENT
-{
-    COMPONENT_UNKNOWN = 0,
-    COMPONENT_LibVA,
-    COMPONENT_EMULATION,
-    COMPONENT_CM,
-    COMPONENT_Encode,
-    COMPONENT_Decode,
-    COMPONENT_VPCommon,
-    COMPONENT_VPreP,
-    COMPONENT_CP,
-    COMPONENT_MEMDECOMP,
-    COMPONENT_MCPY,
-};
-C_ASSERT(COMPONENT_MCPY == 10); // When adding, update assert
-
 //!
 //! \brief Structure to OS sync parameters
 //!
@@ -528,11 +503,14 @@ struct _MOS_GPUCTX_CREATOPTIONS
         uint32_t SSEUValue;
     };
 
+    uint8_t isRealTimePriority;  // 1 if context is created from real time priority command queue (run GT at higher frequency)
+
     _MOS_GPUCTX_CREATOPTIONS() : CmdBufferNumScale(MOS_GPU_CONTEXT_CREATE_DEFAULT),
         RAMode(0),
         ProtectMode(0),
         gpuNode(0),
-        SSEUValue(0){}
+        SSEUValue(0),
+        isRealTimePriority(0){}
 
     virtual ~_MOS_GPUCTX_CREATOPTIONS(){}
 };
@@ -573,7 +551,7 @@ private:
     static std::shared_ptr<GpuCmdResInfoDump> m_instance;
     mutable uint32_t         m_cnt         = 0;
     bool                     m_dumpEnabled = false;
-    std::string              m_path;
+    std::string              m_path        = "";
 };
 #endif // MOS_COMMAND_RESINFO_DUMP_SUPPORTED
 
@@ -586,76 +564,77 @@ struct _MOS_INTERFACE;
 class MosVeInterface;
 class CommandList;
 class CmdBufMgrNext;
+class MosCpInterface;
 
 struct MosStreamState
 {
-    OsDeviceContext   *osDeviceContext = nullptr;
-    GPU_CONTEXT_HANDLE currentGpuContextHandle = MOS_GPU_CONTEXT_INVALID_HANDLE;
-    MOS_COMPONENT      component;
+    OsDeviceContext     *osDeviceContext        = nullptr;
+    GPU_CONTEXT_HANDLE  currentGpuContextHandle = MOS_GPU_CONTEXT_INVALID_HANDLE;
+    MOS_COMPONENT       component               = COMPONENT_UNKNOWN;
 
-    CommandList        *currentCmdList      = nullptr;  //<! Command list used in async mode
-    CmdBufMgrNext      *currentCmdBufMgr    = nullptr;  //<! Cmd buffer manager used in async mode
-    bool                postponedExecution  = false;    //!< Indicate if the stream is work in postponed execution mode. This flag is only used in aync mode.
+    CommandList        *currentCmdList          = nullptr;  //<! Command list used in async mode
+    CmdBufMgrNext      *currentCmdBufMgr        = nullptr;  //<! Cmd buffer manager used in async mode
+    bool                postponedExecution      = false;    //!< Indicate if the stream is work in postponed execution mode. This flag is only used in aync mode.
 
-    bool supportVirtualEngine = false; //!< Flag to indicate using virtual engine interface
-    MosVeInterface *virtualEngineInterface = nullptr; //!< Interface to virtual engine state
-    bool useHwSemaForResSyncInVe = false; //!< Flag to indicate if UMD need to send HW sema cmd under this OS when there is a resource sync need with Virtual Engine interface
-    bool veEnable = false;                //!< Flag to indicate virtual engine enabled (Can enable VE without using virtual engine interface)
-    bool phasedSubmission = false;        //!< Flag to indicate if secondary command buffers are submitted together or separately due to different OS
-    bool frameSplit = true;               //!< Flag to indicate if frame split is enabled (only active when phasedSubmission is true)
-    int32_t hcpDecScalabilityMode = 0;   //!< Hcp scalability mode
-    int32_t veboxScalabilityMode  = 0;    //!< Vebox scalability mode
+    bool supportVirtualEngine                   = false;    //!< Flag to indicate using virtual engine interface
+    MosVeInterface *virtualEngineInterface      = nullptr;  //!< Interface to virtual engine state
+    bool useHwSemaForResSyncInVe                = false;    //!< Flag to indicate if UMD need to send HW sema cmd under this OS when there is a resource sync need with Virtual Engine interface
+    bool veEnable                               = false;    //!< Flag to indicate virtual engine enabled (Can enable VE without using virtual engine interface)
+    bool phasedSubmission                       = false;    //!< Flag to indicate if secondary command buffers are submitted together or separately due to different OS
+    bool frameSplit                             = true;     //!< Flag to indicate if frame split is enabled (only active when phasedSubmission is true)
+    int32_t hcpDecScalabilityMode               = 0;        //!< Hcp scalability mode
+    int32_t veboxScalabilityMode                = 0;        //!< Vebox scalability mode
 
-    bool ctxBasedScheduling = false;  //!< Indicate if context based scheduling is enabled in this stream
-    bool multiNodeScaling = false;    //!< Flag to indicate if multi-node scaling is enabled for virtual engine (only active when ctxBasedScheduling is true)
+    bool ctxBasedScheduling                     = false;    //!< Indicate if context based scheduling is enabled in this stream
+    bool multiNodeScaling                       = false;    //!< Flag to indicate if multi-node scaling is enabled for virtual engine (only active when ctxBasedScheduling is true)
 
-    int32_t ctxPriority = 0;
-    bool softReset = false;  //!< trigger soft reset
+    int32_t ctxPriority                         = 0;
+    bool softReset                              = false;    //!< trigger soft reset
 
-    MosCpInterface *osCpInterface = nullptr; //!< CP interface
+    MosCpInterface *osCpInterface               = nullptr;  //!< CP interface
 
-    bool mediaReset    = false;  //!< Flag to indicate media reset is enabled
+    bool mediaReset                             = false;    //!< Flag to indicate media reset is enabled
 
-    bool simIsActive = false;  //!< Flag to indicate if Simulation is enabled
-    MOS_NULL_RENDERING_FLAGS nullHwAccelerationEnable = {};     //!< To indicate which components to enable Null HW support
+    bool simIsActive                            = false;    //!< Flag to indicate if Simulation is enabled
+    MOS_NULL_RENDERING_FLAGS nullHwAccelerationEnable = {}; //!< To indicate which components to enable Null HW support
 
-    bool usesPatchList = false;               //!< Uses patch list instead of graphic address directly
-    bool usesGfxAddress = false;              //!< Uses graphic address directly instead of patch list
-    bool enableKmdMediaFrameTracking = false; //!< Enable KMD Media frame tracking
-    bool usesCmdBufHeaderInResize = false;    //!< Use cmd buffer header in resize
-    bool usesCmdBufHeader = false;            //!< Use cmd buffer header
+    bool usesPatchList                          = false;    //!< Uses patch list instead of graphic address directly
+    bool usesGfxAddress                         = false;    //!< Uses graphic address directly instead of patch list
+    bool enableKmdMediaFrameTracking            = false;    //!< Enable KMD Media frame tracking
+    bool usesCmdBufHeaderInResize               = false;    //!< Use cmd buffer header in resize
+    bool usesCmdBufHeader                       = false;    //!< Use cmd buffer header
 
     // GPU Reset Statistics
-    uint32_t gpuResetCount      = 0;
-    uint32_t gpuActiveBatch     = 0;
-    uint32_t gpuPendingBatch    = 0;
+    uint32_t gpuResetCount                      = 0;
+    uint32_t gpuActiveBatch                     = 0;
+    uint32_t gpuPendingBatch                    = 0;
 
 #if MOS_COMMAND_BUFFER_DUMP_SUPPORTED
     // Command buffer dump
-    bool  dumpCommandBuffer = false;                      //!< Flag to indicate if Dump command buffer is enabled
-    bool  dumpCommandBufferToFile = false;                //!< Indicates that the command buffer should be dumped to a file
-    bool  dumpCommandBufferAsMessages = false;            //!< Indicates that the command buffer should be dumped via MOS normal messages
-    char  sDirName[MOS_MAX_HLT_FILENAME_LEN] = {0};       //!< Dump Directory name - maximum 260 bytes length
+    bool  dumpCommandBuffer                     = false;    //!< Flag to indicate if Dump command buffer is enabled
+    bool  dumpCommandBufferToFile               = false;    //!< Indicates that the command buffer should be dumped to a file
+    bool  dumpCommandBufferAsMessages           = false;    //!< Indicates that the command buffer should be dumped via MOS normal messages
+    char  sDirName[MOS_MAX_HLT_FILENAME_LEN]    = {0};      //!< Dump Directory name - maximum 260 bytes length
 #endif // MOS_COMMAND_BUFFER_DUMP_SUPPORTED
 
 #if _DEBUG || _RELEASE_INTERNAL
-    bool  enableDbgOvrdInVirtualEngine = false; //!< enable debug override in virtual engine
+    bool  enableDbgOvrdInVirtualEngine          = false;    //!< enable debug override in virtual engine
 
-    int32_t eForceVdbox = 0;  //!< Force select Vdbox
-    int32_t eForceVebox = 0;  //!< Force select Vebox
+    int32_t eForceVdbox                         = 0;        //!< Force select Vdbox
+    int32_t eForceVebox                         = 0;        //!< Force select Vebox
 #endif // _DEBUG || _RELEASE_INTERNAL
 
-    bool  bGucSubmission     = false;  //!< Flag to indicate if guc submission is enabled
+    bool  bGucSubmission                        = false;    //!< Flag to indicate if guc submission is enabled
     OS_PER_STREAM_PARAMETERS  perStreamParameters = nullptr; //!< Parameters of OS specific per stream
 
-    static void *pvSoloContext;  //!< pointer to MediaSolo context
+    static void *pvSoloContext;                             //!< pointer to MediaSolo context
 
 #if MOS_MEDIASOLO_SUPPORTED
 
-    int32_t  bSupportMediaSoloVirtualEngine = 0;  //!< Flag to indicate if MediaSolo uses VE solution in cmdbuffer submission.
-    uint32_t dwEnableMediaSoloFrameNum = 0;       //!< The frame number at which MediaSolo will be enabled, 0 is not valid.
-    int32_t  bSoloInUse = 0;                      //!< Flag to indicate if MediaSolo is enabled
-#endif                                       // MOS_MEDIASOLO_SUPPORTED
+    int32_t  bSupportMediaSoloVirtualEngine     = 0;        //!< Flag to indicate if MediaSolo uses VE solution in cmdbuffer submission.
+    uint32_t dwEnableMediaSoloFrameNum          = 0;        //!< The frame number at which MediaSolo will be enabled, 0 is not valid.
+    int32_t  bSoloInUse                         = 0;        //!< Flag to indicate if MediaSolo is enabled
+#endif  // MOS_MEDIASOLO_SUPPORTED
 
 };
 
@@ -675,6 +654,7 @@ typedef void *              DDI_DEVICE_CONTEXT;  //!< stand for different os spe
 typedef void *              MOS_INTERFACE_HANDLE;
 
 class GpuContextMgr;
+
 //!
 //! \brief Structure to Unified HAL OS resources
 //!
@@ -685,16 +665,17 @@ typedef struct _MOS_INTERFACE
 
     // Saved OS context
     PMOS_CONTEXT                    pOsContext;
-    MOS_GPU_CONTEXT                 CurrentGpuContextOrdinal;
+    MOS_GPU_CONTEXT                 CurrentGpuContextOrdinal = MOS_GPU_CONTEXT_MAX;
     //!< An internal handle that indexes into the list of GPU Context object
     uint32_t                        CurrentGpuContextHandle;
     //!< A handle to the graphics context device that can be used to calls back
     //!< into the kernel subsystem
     HANDLE                          CurrentGpuContextRuntimeHandle;
 
-    //!< Only used in async mode for backward compatiable
+    //!< Only used in async and softlet mos mode for backward compatiable
     GPU_CONTEXT_HANDLE              m_GpuContextHandleMap[MOS_GPU_CONTEXT_MAX] = {0};
-
+    GPU_CONTEXT_HANDLE              m_encContext;
+    GPU_CONTEXT_HANDLE              m_pakContext;
     // OS dependent settings, flags, limits
     int32_t                         b64bit;
     int32_t                         bDeallocateOnExit;
@@ -806,6 +787,10 @@ typedef struct _MOS_INTERFACE
     MOS_STATUS (* pfnDestroyGpuContext) (
         PMOS_INTERFACE              pOsInterface,
         MOS_GPU_CONTEXT             GpuContext);
+    
+    MOS_STATUS (* pfnDestroyGpuContextByHandle) (
+        PMOS_INTERFACE              pOsInterface,
+        GPU_CONTEXT_HANDLE          gpuContextHandle);
 
     MOS_STATUS (* pfnDestroyGpuComputeContext) (
         PMOS_INTERFACE              osInterface,
@@ -824,9 +809,6 @@ typedef struct _MOS_INTERFACE
         GPU_CONTEXT_HANDLE gpuContextHandle,
         MOS_GPU_CONTEXT    GpuContext);
 
-    GpuContextMgr* (*pfnGetGpuContextMgr) (
-        PMOS_INTERFACE     pOsInterface);
-
 #if (_RELEASE_INTERNAL || _DEBUG)
 #if defined(CM_DIRECT_GUC_SUPPORT)
     MOS_STATUS(*pfnDestroyGuC) (
@@ -914,10 +896,6 @@ typedef struct _MOS_INTERFACE
         MOS_GPU_CONTEXT             busyGPUCtx,
         MOS_GPU_CONTEXT             requestorGPUCtx);
 
-    void(* pfnSyncWith3DContext)(
-        PMOS_INTERFACE              pOsInterface,
-        PMOS_SYNC_PARAMS            pParams);
-
     MOS_STATUS (* pfnRegisterBBCompleteNotifyEvent) (
         PMOS_INTERFACE              pOsInterface,
         MOS_GPU_CONTEXT             GpuContext);
@@ -1120,10 +1098,6 @@ typedef struct _MOS_INTERFACE
         PMOS_INTERFACE              pOsInterface);
 #endif // MOS_MEDIASOLO_SUPPORTED
 
-    MOS_STATUS (* pfnWaitOnResource) (
-        PMOS_INTERFACE              pOsInterface,
-        PMOS_RESOURCE               pResource);
-
     uint32_t (* pfnGetGpuStatusTag) (
         PMOS_INTERFACE              pOsInterface,
         MOS_GPU_CONTEXT             GpuContext);
@@ -1183,15 +1157,6 @@ typedef struct _MOS_INTERFACE
         PMOS_INTERFACE              pOsInterface,
         PMOS_COMMAND_BUFFER         pCmdBuffer);
 
-    MOS_FORMAT (* pfnFmt_OsToMos) (
-        MOS_OS_FORMAT               format);
-
-    MOS_OS_FORMAT (* pfnFmt_MosToOs) (
-        MOS_FORMAT                  format);
-
-    GMM_RESOURCE_FORMAT (* pfnFmt_MosToGmm) (
-        MOS_FORMAT                  format);
-
     void (* pfnSetPerfTag) (
         PMOS_INTERFACE              pOsInterface,
         uint32_t                    PerfTag);
@@ -1228,9 +1193,6 @@ typedef struct _MOS_INTERFACE
     MOS_STATUS (*pfnFreeLibrary) (
         HINSTANCE                   hInstance);
 
-    void (*pfnLogData)(
-        char                        *pData);
-
     MOS_NULL_RENDERING_FLAGS  (* pfnGetNullHWRenderFlags) (
         PMOS_INTERFACE              pOsInterface);
 
@@ -1292,22 +1254,12 @@ typedef struct _MOS_INTERFACE
         PMOS_INTERFACE              pOsInterface,
         MOS_GPU_NODE                VideoNodeOrdinal);
 
-    int32_t (*pfnSetCpuCacheability)(
-        PMOS_INTERFACE              pOsInterface,
-        PMOS_ALLOC_GFXRES_PARAMS    pParams);
-
     MOS_STATUS(*pfnSkipResourceSync)(
         PMOS_RESOURCE               pOsResource);
 
     MOS_STATUS(*pfnSetObjectCapture)(
         PMOS_RESOURCE               pOsResource);
 
-    MOS_STATUS(*pfnSkipResourceSyncDynamic)(
-        PMOS_RESOURCE               pOsResource);
-
-    MOS_STATUS(*pfnEnableResourceSyncDynamic)(
-        PMOS_RESOURCE               pOsResource);
-
     bool(*pfnIsValidStreamID)(
         PMOS_RESOURCE               pOsResource);
 
@@ -1325,12 +1277,27 @@ typedef struct _MOS_INTERFACE
     int32_t (*pfnIsGPUHung)(
         PMOS_INTERFACE              pOsInterface);
 
-    MOS_STATUS (*pfnDumpTraceGpuData)(
+    bool (*pfnIsMultipleCodecDevicesInUse)(
+        PMOS_INTERFACE              pOsInterface);
+
+    MOS_GPU_NODE(*pfnGetLatestVirtualNode)(
         PMOS_INTERFACE              pOsInterface,
-        const char *                pcName,
-        uint32_t                    flags,
-        PMOS_RESOURCE               pOsResource,
-        uint32_t                    dwSize);
+        MOS_COMPONENT               component);
+
+    void (*pfnSetLatestVirtualNode)(
+        PMOS_INTERFACE              pOsInterface,
+        MOS_GPU_NODE                node);
+
+    MOS_GPU_NODE (*pfnGetDecoderVirtualNodePerStream)(
+        PMOS_INTERFACE pOsInterface);
+
+    void (*pfnSetDecoderVirtualNodePerStream)(
+        PMOS_INTERFACE pOsInterface,
+        MOS_GPU_NODE   node);
+
+    HANDLE (*pfnGetUmdDmaCompleteEventHandle)(
+        PMOS_INTERFACE  osInterface,
+        MOS_GPU_CONTEXT gpuContext);
 
     //!
     //! \brief    Get Aux Table base address
@@ -1417,6 +1384,160 @@ typedef struct _MOS_INTERFACE
     void (*pfnNotifyStreamIndexSharing)(
         PMOS_INTERFACE              pOsInterface);
 
+    //!
+    //! \brief    Check if mismatch order programming model supported in current device
+    //!
+    //! \return   bool
+    //!
+    bool (*pfnIsMismatchOrderProgrammingSupported)();
+
+    //! \brief    Unified OS add command to command buffer
+    //! \details  Offset returned is dword aligned but size requested can be byte aligned
+    //! \param    PMOS_COMMAND_BUFFER pCmdBuffer
+    //!           [in/out] Pointer to Command Buffer
+    //! \param    void  *pCmd
+    //!           [in] Command Pointer
+    //! \param    uint32_t dwCmdSize
+    //!           [in] Size of command in bytes
+    //! \return   MOS_STATUS
+    //!           Return MOS_STATUS_SUCCESS if successful, otherwise failed
+    //!
+    MOS_STATUS (*pfnAddCommand)(
+        PMOS_COMMAND_BUFFER pCmdBuffer,
+        const void          *pCmd,
+        uint32_t            dwCmdSize);
+
+    //!
+    //! \brief    Check virtual engine is supported
+    //! \details  Check virtual engine is supported
+    //! \param    PMOS_INTERFACE pOsInterface
+    //!           [in] OS Interface
+    //! \return   MOS_STATUS
+    //!           MOS_STATUS_SUCCESS if succeeded, otherwise error code
+    //!
+    MOS_STATUS (*pfnVirtualEngineSupported)(
+        PMOS_INTERFACE osInterface,
+        bool           isDecode,
+        bool           veDefaultEnable);
+
+    //!
+    //! \brief    Retrieve the CachePolicyMemoryObject for a resource
+    //! \details  Retrieve the CachePolicyMemoryObject for a resource
+    //! \param    PMOS_INTERFACE osInterface
+    //!           [in] OS Interface
+    //! \param    PMOS_RESOURCE resource
+    //!           [in] resource
+    //! \return   MEMORY_OBJECT_CONTROL_STATE
+    //!           return a value of MEMORY_OBJECT_CONTROL_STATE
+    //!
+    MEMORY_OBJECT_CONTROL_STATE (*pfnGetResourceCachePolicyMemoryObject)(
+        PMOS_INTERFACE      osInterface,
+        PMOS_RESOURCE       resource);
+
+    //!
+    //! \brief    Get Buffer Type
+    //! \details  Returns the type of buffer, 1D, 2D or volume
+    //! \param    PMOS_RESOURCE pOsResource
+    //!           [in] Pointer to OS Resource
+    //! \return   GFX resource Type
+    //!
+    MOS_GFXRES_TYPE (*pfnGetResType)(
+        PMOS_RESOURCE pOsResource);
+
+    //!
+    //! \brief    Get TimeStamp frequency base
+    //! \details  Get TimeStamp frequency base from OsInterface
+    //! \param    PMOS_INTERFACE pOsInterface
+    //!           [in] OS Interface
+    //! \return   uint32_t
+    //!           time stamp frequency base
+    //!
+    uint32_t (*pfnGetTsFrequency)(
+        PMOS_INTERFACE         pOsInterface);
+
+    //!
+    //! \brief    Set hint parameters
+    //! \details  
+    //! \param    PMOS_INTERFACE pOsInterface
+    //!           [in] Pointer to OS interface structure
+    //! \param    MOS_VIRTUALENGINE_SET_PARAMS veParams
+    //!           [out] VIRTUALENGINE SET PARAMS
+    //! \return   MOS_STATUS
+    //!           MOS_STATUS_SUCCESS if succeeded, otherwise error code
+    //!
+    MOS_STATUS (*pfnSetHintParams)(
+        PMOS_INTERFACE                pOsInterface,
+        PMOS_VIRTUALENGINE_SET_PARAMS veParams);
+
+    //!
+    //! \brief    Checks whether the requested resource is releasable
+    //! \param    PMOS_INTERFACE pOsInterface
+    //!           [in] OS Interface
+    //! \param    PMOS_RESOURCE pOsResource
+    //!           [in] Pointer to OS Resource
+    //! \return   MOS_STATUS
+    //!           MOS_STATUS_SUCCESS if requested can be released, otherwise MOS_STATUS_UNKNOWN
+    //!
+    MOS_STATUS (*pfnIsResourceReleasable)(
+        PMOS_INTERFACE         pOsInterface,
+        PMOS_RESOURCE          pOsResource);
+
+    //!
+    //! \brief    Virtual Engine Init for media Scalability
+    //! \details  
+    //! \param    PMOS_INTERFACE pOsInterface
+    //!           [in] Pointer to OS interface structure
+    //! \param    PMOS_VIRTUALENGINE_HINT_PARAMS veHitParams
+    //!           [out] Pointer to Virtual Engine hint parameters
+    //! \param    PMOS_VIRTUALENGINE_INTERFACE veInterface
+    //!           [out] Pointer to Virtual Engine Interface
+    //! \return   MOS_STATUS
+    //!           MOS_STATUS_SUCCESS if succeeded, otherwise error code
+    //!
+    MOS_STATUS (*pfnVirtualEngineInit)(
+        PMOS_INTERFACE                  pOsInterface,
+        PMOS_VIRTUALENGINE_HINT_PARAMS* veHitParams,
+        MOS_VIRTUALENGINE_INIT_PARAMS&  veInParams);
+    //!
+    //! \brief    initialize virtual engine interface
+    //! \details  initialize virtual engine interface
+    //! \param    [in]  PMOS_INTERFACE
+    //!                pointer to mos interface
+    //! \param    [in]  PMOS_VIRTUALENGINE_INIT_PARAMS pVEInitParms
+    //!                pointer to VE init parameters
+    //! \return   MOS_STATUS
+    //!           MOS_STATUS_SUCCESS if success, else fail reason
+    //!
+    MOS_STATUS (*pfnVirtualEngineInterfaceInitialize)(
+        PMOS_INTERFACE                    pOsInterface,
+        PMOS_VIRTUALENGINE_INIT_PARAMS    pVEInitParms);
+
+    //!
+    //! \brief    Destroy veInterface
+    //! \details  
+    //! \param    PMOS_VIRTUALENGINE_INTERFACE *veInterface
+    //!           [in] Pointer to PMOS_VIRTUALENGINE_INTERFACE
+    //! \return   MOS_STATUS
+    //!           MOS_STATUS_SUCCESS if succeeded, otherwise error code
+    //!
+    MOS_STATUS (*pfnDestroyVeInterface)(
+        PMOS_VIRTUALENGINE_INTERFACE *veInterface);
+
+#if (_DEBUG || _RELEASE_INTERNAL)
+    //!
+    //! \brief    gpuCtxCreateOption Init for media Scalability
+    //! \details  
+    //! \param    PMOS_INTERFACE pOsInterface
+    //!           [in] Pointer to OS interface structure
+    //! \param    uint8_t id
+    //!           [out] EngineLogicId
+    //! \return   MOS_STATUS
+    //!           MOS_STATUS_SUCCESS if succeeded, otherwise error code
+    //!
+    MOS_STATUS (*pfnGetEngineLogicId)(
+        PMOS_INTERFACE                 pOsInterface,
+        uint8_t&                       id);
+#endif
     //!
     //! \brief   Get User Setting instance
     //!
@@ -1461,6 +1582,8 @@ typedef struct _MOS_INTERFACE
 
     bool                            bOptimizeCpuTiming;                           //!< Optimize Cpu Timing
 
+    bool                            bNullHwIsEnabled;                             //!< Null Hw is enabled or not
+
     //!< os interface extension
     void                            *pOsExt;
 } MOS_INTERFACE;
@@ -1480,7 +1603,7 @@ extern "C" {
 //!
 MOS_STATUS Mos_InitInterface(
     PMOS_INTERFACE      pOsInterface,
-    PMOS_CONTEXT        pOsDriverContext,
+    MOS_CONTEXT_HANDLE  pOsDriverContext,
     MOS_COMPONENT       component);
 
 //! \brief    Destroy the mos interface
@@ -1613,6 +1736,20 @@ MOS_STATUS Mos_CheckVirtualEngineSupported(
     bool           isDecode,
     bool           veDefaultEnable);
 
+//!
+//! \brief    Retrieve the CachePolicyMemoryObject for a resource
+//! \details  Retrieve the CachePolicyMemoryObject for a resource
+//! \param    PMOS_INTERFACE osInterface
+//!           [in] OS Interface
+//! \param    PMOS_RESOURCE resource
+//!           [in] resource
+//! \return   MEMORY_OBJECT_CONTROL_STATE
+//!           return a value of MEMORY_OBJECT_CONTROL_STATE
+//!
+MEMORY_OBJECT_CONTROL_STATE Mos_GetResourceCachePolicyMemoryObject(
+    PMOS_INTERFACE      osInterface,
+    PMOS_RESOURCE       resource);
+
 struct ContextRequirement
 {
     bool IsEnc               = false;
