@@ -29,6 +29,162 @@
 #ifndef __MOS_OS_TRACE_H__
 #define __MOS_OS_TRACE_H__
 
+//!
+//! \brief helper to expand trace event data from dynmac size Marco
+//!
+
+#define _TR_PRE(x) x
+#define _TR_CAT_(a, b) a##b
+#define _TR_CAT(a, b) _TR_CAT_(a, b)
+
+//!
+//! \brief internal marco to calc the number of args in __VA_ARGS__, support count upto 10 args
+//! Due to C99 compiler definition, _TR_COUNT_() return 1. need new marco to use.
+//!
+#define _TR_CNT_N(_1, _2, _3, _4, _5, _6, _7, _8, _9, _10, cnt, ...) cnt
+#define _TR_COUNT(...) _TR_PRE(_TR_CNT_N(__VA_ARGS__, 10, 9, 8, 7, 6, 5, 4, 3, 2, 1, 0))
+
+//!
+//! \brief internal marco to calc byte size of all args, and memcpy to pre-defined buffer
+//!
+#define _TR_OPsize(x)  +sizeof(x)
+#define _TR_OPfield(x) {auto _t=x;memcpy(_buf + _i, &(_t), sizeof(x));_i += sizeof(x);}
+
+//!
+//! \brief internal marco to calc byte size of all args, and memcpy to pre-defined buffer
+//!
+#define _TR_EXPAND1(o, x, ...) _TR_PRE(_TR_CAT(_TR_OP, o)(x))
+#define _TR_EXPAND2(o, x, ...) _TR_EXPAND1(o, x) _TR_PRE(_TR_EXPAND1(o, __VA_ARGS__))
+#define _TR_EXPAND3(o, x, ...) _TR_EXPAND1(o, x) _TR_PRE(_TR_EXPAND2(o, __VA_ARGS__))
+#define _TR_EXPAND4(o, x, ...) _TR_EXPAND1(o, x) _TR_PRE(_TR_EXPAND3(o, __VA_ARGS__))
+#define _TR_EXPAND5(o, x, ...) _TR_EXPAND1(o, x) _TR_PRE(_TR_EXPAND4(o, __VA_ARGS__))
+#define _TR_EXPAND6(o, x, ...) _TR_EXPAND1(o, x) _TR_PRE(_TR_EXPAND5(o, __VA_ARGS__))
+#define _TR_EXPAND7(o, x, ...) _TR_EXPAND1(o, x) _TR_PRE(_TR_EXPAND6(o, __VA_ARGS__))
+#define _TR_EXPAND8(o, x, ...) _TR_EXPAND1(o, x) _TR_PRE(_TR_EXPAND7(o, __VA_ARGS__))
+#define _TR_EXPAND9(o, x, ...) _TR_EXPAND1(o, x) _TR_PRE(_TR_EXPAND8(o, __VA_ARGS__))
+#define _TR_EXPAND10(o, x, ...) _TR_EXPAND1(o, x) _TR_PRE(_TR_EXPAND9(o, __VA_ARGS__))
+// Note: increase _TR_EXPANDxx to support more param
+
+#define _TR_PARAM_SIZE(...) _TR_PRE(_TR_CAT(_TR_EXPAND, _TR_COUNT(__VA_ARGS__))(size, __VA_ARGS__))
+#define _TR_PARAM_FIELD(...) _TR_PRE(_TR_CAT(_TR_EXPAND, _TR_COUNT(__VA_ARGS__))(field, __VA_ARGS__))
+
+//!
+//! \brief trace marco interface, for external usage, split 2 marco for customzed trace write interface
+//!
+#define TR_FILL_PARAM(...)                       \
+    int  _i = 0;                                 \
+    char _buf[0 _TR_PARAM_SIZE(__VA_ARGS__)];    \
+    _TR_PARAM_FIELD(__VA_ARGS__);
+
+#define TR_WRITE_PARAM(func, id, op)             \
+    func(id, op, _buf, sizeof(_buf), nullptr, 0);
+
+//!
+//! \brief Keyword for ETW tracing, 1bit per keyworld, total 64bits
+//!
+typedef enum _MEDIA_EVENT_FILTER_KEYID
+{
+    TR_KEY_DECODE_PICPARAM = 0,
+    TR_KEY_DECODE_SLICEPARAM,
+    TR_KEY_DECODE_TILEPARAM,
+    TR_KEY_DECODE_QMATRIX,
+    TR_KEY_DECODE_BITSTREAM_INFO,
+    TR_KEY_DECODE_BITSTREAM,
+    TR_KEY_DECODE_INTERNAL,
+    TR_KEY_DECODE_COMMAND,
+    TR_KEY_DECODE_DSTYUV,
+    TR_KEY_DECODE_REFYUV,
+    TR_KEY_DECODE_MV,
+    TR_KEY_DECODE_SUBSET,
+    TR_KEY_MOSMSG_ALL = 12,
+    TR_KEY_CALL_STACK,
+    TR_KEY_DATA_DUMP  = 16,
+    TR_KEY_MOSMSG_CP,
+    TR_KEY_MOSMSG_VP,
+    TR_KEY_MOSMSG_CODEC,
+    TR_KEY_MOSMSG_DDI,
+    TR_KEY_MOSMSG_MOS,
+    TR_KEY_MOSMSG_MHW,
+    TR_KEY_DECODE_INFO,
+    TR_KEY_ENCODE_EVENT_DDI = 24,
+    TR_KEY_ENCODE_EVENT_API_STICKER,
+    TR_KEY_ENCODE_EVENT_INTERNAL,
+    TR_KEY_ENCODE_DATA_INPUT_SURFACE,
+    TR_KEY_ENCODE_DATA_REF_SURFACE,
+    TR_KEY_ENCODE_DATA_RECON_SURFACE,
+    TR_KEY_ENCODE_DATA_BITSTREAM,
+    TR_KEY_DECODE_DSTYUV_IN_TRACE,
+} MEDIA_EVENT_FILTER_KEYID;
+
+#pragma pack(push, 8)
+struct MtSetting
+{
+    struct FastDump
+    {
+        uint8_t  allowDataLoss : 1;
+        uint8_t  frameIdxBasedSampling : 1;
+        uint8_t  memUsagePolicy : 2;
+        uint8_t  writeMode : 2;
+        uint8_t  informOnError : 1;
+        uint8_t  rsv0 : 1;
+        uint8_t  rsv1;
+        uint8_t  maxPrioritizedMem;
+        uint8_t  maxDeprioritizedMem;
+        uint8_t  weightRenderCopy;
+        uint8_t  weightVECopy;
+        uint8_t  weightBLTCopy;
+        uint8_t  rsv2;
+        uint64_t samplingTime;
+        uint64_t samplingInterval;
+        uint64_t bufferSize;
+        char     filePath[1024];
+    } fastDump;
+
+    uint8_t rsv[2528];
+};
+
+// 4KB in total
+struct MtControlData
+{
+    uint32_t enable;
+    uint8_t  level;
+    uint8_t  rsv[3];
+    uint64_t filter[63];
+
+    MtSetting setting;
+};
+#pragma pack(pop)
+
+enum class MT_EVENT_LEVEL
+{
+    ALWAYS,
+    CRITICAL,
+    ERR,
+    WARNING,
+    INFO,
+    VERBOSE,
+};
+
+enum class MT_DATA_LEVEL
+{
+    FIRST_64B,  // dump first min(DataSize, 64) bytes
+    QUARTER,    // dump first min(DataSize, max(DataSize/4, 64)) bytes
+    HALF,       // dump first min(DataSize, max(DataSize/2, 64)) bytes
+    FULL,       // dump all data
+};
+
+enum class MT_LOG_LEVEL
+{
+    ALWAYS,
+    CRITICAL,
+    NORMAL,
+    VERBOSE,
+    FUNCTION_ENTRY,
+    FUNCTION_EXIT,
+    FUNCTION_ENTRY_VERBOSE,
+    MEMNINJA,
+};
+
 typedef enum _MEDIA_EVENT
 {
     UNDEFINED_EVENT = 0,            //! reserved id, should not used in driver
@@ -172,8 +328,8 @@ typedef enum _MEDIA_EVENT
     EVENT_DECODE_BUFFER_PICPARAM_VP9,              //! event for Decode VP9 Pic Paramters
     EVENT_DECODE_BUFFER_SEGPARAM_VP9,              //! event for Decode VP9 Segment Paramters
     EVENT_DECODE_BUFFER_SLICEPARAM_VP9,            //! event for Decode VP9 Slice Paramters
-    EVENT_DECODE_BUFFER_Bitstream,                 //! event for Decode Bitstream
-    EVENT_DECODE_CMD_HCP_PIPEMODESELECT,           //! event for Decode HcpPipeModeSelect Cmd
+    EVENT_DECODE_INFO_BITSTREAM,                   //! event for Decode Bitstream Info
+    EVENT_DECODE_INFO_PICTURE,                     //! event for Decode Picture Info
     EVENT_DECODE_CMD_HCP_SURFACESTATE,             //! event for Decode HcpSurfaceState Cmd
     EVENT_DECODE_CMD_HCP_PIPEBUFADDRSTATE,         //! event for Decode HcpPipeBufAddrState Cmd
     EVENT_DECODE_CMD_HCP_INDOBJBASEADDRSTATE,      //! event for Decode HcpIndObjBaseAddrState Cmd
@@ -190,15 +346,47 @@ typedef enum _MEDIA_EVENT
     EVENT_DECODE_BUFFER_REXTPICPARAM_HEVC,         //! event for Decode HEVC REXT Pic Paramters
     EVENT_DECODE_BUFFER_SCCPICPARAM_HEVC,          //! event for Decode HEVC SCC Pic Paramters
     EVENT_DECODE_BUFFER_SLICEPARAM_HEVC,           //! event for Decode HEVC Slice Paramters
-    EVENT_DECODE_CMD_HCP_PICSTATE_HEVC,            //! event for Decode HcpHevcPicState Cmd
-    EVENT_DECODE_CMD_HCP_REXTPICSTATE_HEVC,        //! event for Decode HcpHevcRextPicState Cmd
-    EVENT_DECODE_CMD_HCP_SCCPICSTATE_HEVC,         //! event for Decode HcpHevcSccPicState Cmd
+    EVENT_DECODE_BUFFER_LONGSLICEPARAM_HEVC,       //! event for Decode HEVC Long Slice Paramters
+    EVENT_DECODE_BUFFER_REXTLONGSLICEPARAM_HEVC,   //! event for Decode HEVC RExt Long Slice Paramters
+    EVENT_DECODE_BUFFER_LONGSLICEPARAM_AVC,        //! event for Decode AVC Long Slice Paramters
     EVENT_DECODE_FEATURE_MMC,                      //! event for Decode Feature MMC
     EVENT_DECODE_FEATURE_VT_SCALABILITY,           //! event for Decode Feature Virtual Tile Scalability
     EVENT_DECODE_FEATURE_RT_SCALABILITY,           //! event for Decode Feature Real Tile Scalability
     EVENT_DECODE_FEATURE_DECODEMODE_REPORT,        //! event for Decode Feature Decode Mode Report
-    EVENT_DECODE_DST_DUMPINFO,                     //! event for Decode Dst Surface Info
-    EVENT_DECODE_REF_DUMPINFO,                     //! event for Decode Ref Surface Info
+    EVENT_DECODE_DUMPINFO_DST,                     //! event for Decode Dst Dump Info
+    EVENT_DECODE_DUMPINFO_REF,                     //! event for Decode Ref Dump Info
+    EVENT_CALL_STACK,                              //! event for call stack dump
+    EVENT_ENCODE_DDI_11_CREATEVIDEOENCODER,        //! event for Encode Create Device
+    EVENT_ENCODE_DDI_11_ENCODEFRAME,               //! event for Encode frame, mainly excute
+    EVENT_ENCODE_DDI_11_GETCAPS,                   //! event for Encode getting caps
+    EVENT_ENCODE_DDI_11_GETPROFILECOUNT,           //! event for Encode get profile count
+    EVENT_ENCODE_DDI_11_GETPROFILE,                //! event for Encode get profile
+    EVENT_ENCODE_DDI_11_CHECKFORMAT,               //! event for Encode check format
+    EVENT_ENCODE_DDI_11_GETCONFIGCOUNT,            //! event for Encode get config count
+    EVENT_ENCODE_DDI_11_GETCONFIG,                 //! event for Encode get config
+    EVENT_ENCODE_DDI_STATUS_REPORT_HEVC,           //! event for HEVC encode status report
+    EVENT_ENCODE_DDI_SLICE_STATUS_REPORT_HEVC,     //! event for HEVC encode slice status report
+    EVENT_ENCODE_DDI_EXT_STATUS_REPORT_HEVC,       //! event for HEVC encode ext status report
+    EVENT_ENCODE_DDI_CAPS_HEVC,                    //! event for HEVC encode caps
+    EVENT_ENCODE_DDI_SEQ_PARAM_HEVC,               //! event for HEVC encode sequence param
+    EVENT_ENCODE_DDI_PIC_PARAM_HEVC,               //! event for HEVC encode picture param
+    EVENT_ENCODE_DDI_SLC_PARAM_HEVC,               //! event for HEVC encode slice param
+    EVENT_ENCODE_DDI_VERSION_HEVC,                 //! event for HEVC encode DDI version
+    EVENT_ENCODE_API_STICKER_HEVC,                 //! event for HEVC encode API sticker
+    EVENT_DECODE_DDI_DISPLAYINFOVA,                //! event for Decode DDI DisplayInfo
+    EVENT_DECODE_DDI_CREATEBUFFERVA,               //! event for Decode DDI CreateBuffer
+    EVENT_DECODE_DDI_BEGINPICTUREVA,               //! event for Decode DDI BeginPicture
+    EVENT_DECODE_DDI_ENDPICTUREVA,                 //! event for Decode DDI EndPicture
+    EVENT_DECODE_DDI_RENDERPICTUREVA,              //! event for Decode DDI RenderPicture
+    EVENT_DECODE_DDI_CLEARUPVA,                    //! event for Decode DDI ClearUp
+    EVENT_DECODE_DDI_STATUSREPORTVA,               //! event for Decode DDI StatusReport
+    EVENT_DECODE_DDI_CREATECONTEXTVA,              //! event for Decode DDI CreateContext
+    EVENT_DECODE_DDI_DESTROYCONTEXTVA,             //! event for Decode DDI DestroyContext
+    EVENT_DECODE_DDI_GETDECCTXFROMBUFFERIDVA,      //! event for Decode DDI GetDecCtxFromBufferID
+    EVENT_DECODE_DDI_FREEBUFFERHEAPELEMENTSVA,     //! event for Decode DDI FreeBufferHeapElements
+    EVENT_DECODE_DDI_SETGPUPRIORITYVA,             //! event for Decode DDI SetGpuPriority
+    EVENT_DECODE_FEATURE_DECODEMODE_REPORTVA,      //! event for Decode Feature Decode Mode Report
+    EVENT_DECODE_INFO_PICTUREVA,                   //! event for Decode Picture Info VA
 } MEDIA_EVENT;
 
 typedef enum _MEDIA_EVENT_TYPE
@@ -216,6 +404,103 @@ typedef enum _MT_LEVEL
     MT_CRITICAL = 2,  //! critical runtime log
 } MT_LEVEL;
 
+class MtEnable
+{
+public:
+    MtEnable(bool flag = false) : m_flag(flag) {}
+
+    MtEnable(const uint32_t *pFlag) : m_pFlag(pFlag) {}
+
+    ~MtEnable()
+    {
+        Reset();
+    }
+
+    operator bool() const
+    {
+        return m_pFlag ? *(m_pFlag) : m_flag;
+    }
+
+    void Reset()
+    {
+        m_pFlag = nullptr;
+    }
+
+private:
+    bool            m_flag  = false;
+    const uint32_t *m_pFlag = nullptr;
+};
+
+class MtFilter
+{
+public:
+    MtFilter(const uint64_t *filter = nullptr, size_t filterNum = 0) : m_filter(filter), m_maxKeyNum(filterNum * N)
+    {
+        if (filter && filterNum == 0)
+        {
+            m_maxKeyNum = N;
+        }
+    }
+
+    ~MtFilter()
+    {
+        Reset();
+    }
+
+    bool operator()(MEDIA_EVENT_FILTER_KEYID key)
+    {
+        return m_filter && static_cast<size_t>(key) < m_maxKeyNum ? (m_filter[key / N] & (1ULL << (key % N))) : false;
+    }
+
+    void Reset()
+    {
+        m_filter    = nullptr;
+        m_maxKeyNum = 0;
+    }
+
+private:
+    static constexpr size_t N = sizeof(uint64_t) << 3;
+
+    const uint64_t *m_filter;
+    size_t          m_maxKeyNum;
+};
+
+class MtLevel
+{
+public:
+    MtLevel(const uint8_t *level = nullptr) : m_level(reinterpret_cast<const Level *>(level)) {}
+
+    ~MtLevel() { Reset(); }
+
+    bool operator()(MT_EVENT_LEVEL level)
+    {
+        return m_level ? (level <= static_cast<MT_EVENT_LEVEL>(m_level->event)) : false;
+    }
+
+    bool operator()(MT_DATA_LEVEL level)
+    {
+        return m_level ? (level <= static_cast<MT_DATA_LEVEL>(m_level->data)) : false;
+    }
+
+    bool operator()(MT_LOG_LEVEL level)
+    {
+        return m_level ? (level <= static_cast<MT_LOG_LEVEL>(m_level->log)) : false;
+    }
+
+    void Reset() { m_level = nullptr; }
+
+private:
+    struct Level
+    {
+        uint8_t event : 3;  // MT_EVENT_LEVEL
+        uint8_t data : 2;   // MT_DATA_LEVEL
+        uint8_t log : 3;    // MT_LOG_LEVEL
+    };
+
+private:
+    const Level *m_level;
+};
+
 #pragma pack(1)
 typedef struct _MT_PARAM
 {
@@ -249,6 +534,15 @@ typedef enum _MT_LOG_ID
     MT_MOS_SYNC,
     MT_MOS_GPUCXT_SET,
     MT_MOS_GPUCXT_VALID,
+    MT_MOS_GPUVA_MAP,
+    MT_MOS_GPUVA_FREE,
+    MT_MOS_MM_ALLOCATE_EXTRES,
+    MT_MOS_MM_MAKERESIDENT,
+    MT_MOS_MM_UPDATERESIDENCY,
+    MT_MOS_MM_EVICT,
+    MT_MOS_MM_BIND_GPU_RESOURCE_VIRTUAL,
+    MT_MOS_MM_UNBIND_GPU_RESOURCE_VIRTUAL,
+    MT_ERR_CRITICAL_MESSAGE,
     MT_LOG_ID_CP_BASE = 0x01000000,
     MT_CP_HAL_NOT_INITIALIZED,
     MT_CP_HAL_FAIL,
@@ -404,6 +698,7 @@ typedef enum _MT_LOG_ID
     MT_VP_BLT_SEGMENTPARAM,
     MT_VP_BLT_MCPYPARAM,
     MT_VP_USERFEATURE_CTRL,
+    MT_VP_FTR_REPORT,
     MT_VP_HAL_ID_BASE = 0x02000400,
     MT_VP_HAL_PIPELINE_ADAPTER,
     MT_VP_HAL_PIPELINE_ADAPTER_EXT_ENTRY,
@@ -432,6 +727,12 @@ typedef enum _MT_LOG_ID
     MT_VP_HAL_POLICY_GET_OUTPIPECAPS,
     MT_VP_HAL_POLICY_INIT_EXECCAPS,
     MT_VP_HAL_FC_SCALINGINFO,
+    MT_VP_HAL_VESFC_HWLIMIT,
+    MT_VP_HAL_RENDER_SETUP_WALKER_PARAM,
+    MT_VP_HAL_RENDER_SETUP_CURBE_STATE,
+    MT_VP_HAL_POLICY_FLITER_FTR_COMBINE,
+    MT_VP_HAL_FC_UPDATE_COMP_PARAM,
+    MT_VP_HAL_FC_GET_CURBE_STATE,
     MT_VP_MHW_ID_BASE = 0x02002000,
     MT_VP_MHW_VE_SURFSTATE_INPUT,
     MT_VP_MHW_VE_SURFSTATE_OUT,
@@ -444,6 +745,7 @@ typedef enum _MT_LOG_ID
     MT_VP_KERNEL_CSC,
     MT_VP_KERNEL_RULE,
     MT_VP_KERNEL_LIST_ADD,
+    MT_VP_KERNEL_Init,
     MT_MEDIA_COPY_ID_BASE = 0x02004000,
     MT_VE_DECOMP_COPY,
     MT_MEDIA_COPY,
@@ -497,6 +799,23 @@ typedef enum _MT_PARAM_ID
     MT_FUNC_END,
     MT_FUNC_RET,
     MT_VIEW_TYPE,
+    MT_SURF_GMM_PATIDX,
+    MT_SURF_GMM_RESUSAGE,
+    MT_SURF_GMM_GPUVA,
+    MT_SURF_GMM_PAGINGFENCE,
+    MT_SURF_MOS_RESOURCE_USAGE,
+    MT_SURF_ALLOCINFO_PTR,
+    MT_SURF_ALLOCINFO_ISMEDIAINTERNAL,
+    MT_SURF_ALLOCINFO_ISPERSISTENT,
+    MT_SURF_ALLOCINFO_3DRESOURCE_PTR,
+    MT_SURF_ALLOCINFO_ISNEW,
+    MT_SURF_MEDIARESINFO_PTR,
+    MT_SURF_BE_INTERNAL_RESIDENT_MAP,
+    MT_SURF_MAPPED_ALLOCINFO,
+    MT_DEVICE_HANDLE,
+    MT_COMMAND_GPUVA,
+    MT_FUNC_NAME,
+    MT_FUNC_LINE,
     MT_PARAM_ID_MOS_BASE = 0x00001000,
     MT_MOS_STATUS,
     MT_MOS_GPU_NODE,
@@ -509,6 +828,7 @@ typedef enum _MT_PARAM_ID
     MT_MOS_SYNC_HAZARDTYPE,
     MT_MOS_SYNC_BUSYCTX,
     MT_MOS_SYNC_REQCTX,
+    MT_MOS_MM_EXT_RESOURCE_NEED_MAKE_RESIDENT,
     MT_PARAM_ID_CP_BASE  = 0x01000000,
     MT_CP_SESSION_TYPE,
     MT_CP_SESSION_MODE,
@@ -627,6 +947,14 @@ typedef enum _MT_PARAM_ID
     MT_VP_HAL_SAMPLER_INDEX,
     MT_VP_HAL_FC_LAYER,
     MT_VP_HAL_FC_LAYER_SURFENTRY,
+    MT_VP_RENDER_VE_FTRINUSE,
+    MT_VP_RENDER_VE_PREPROC_TCC,
+    MT_VP_RENDER_VE_PREPROC_IEF,
+    MT_VP_HAL_EXECCAPS_FORCE_DI2RENDER,
+    MT_VP_HAL_EUFUSION_BYPASS,
+    MT_VP_HAL_MMCINUSE,
+    MT_VP_HAL_FRC_MODE,
+    MT_VP_HAL_CAPTURE_PIPE,
     MT_PARAM_ID_VP_MHW_BASE = 0x02002000,
     MT_VP_MHW_VE_SCALABILITY_EN,
     MT_VP_MHW_VE_SCALABILITY_USE_SFC,
@@ -642,6 +970,12 @@ typedef enum _MT_PARAM_ID
     MT_MEDIA_COPY_CAPS, 
     MT_MEDIA_COPY_DIRECTION,
     MT_MEDIA_COPY_METHOD,
+    MT_MEDIA_COPY_DEVICE_PTR,
+    MT_MEDIA_COPY_DATASIZE,
+    MT_MEDIA_COPY_PLANE_NUM,
+    MT_MEDIA_COPY_PLANE_PITCH,
+    MT_MEDIA_COPY_PLANE_OFFSET,
+    MT_MEDIA_COPY_LIMITATION,
     MT_PARAM_ID_DEC_BASE = 0x03000000,
     MT_DEC_HUC_ERROR_STATUS2,
     MT_CODEC_HAL_MODE,
