@@ -159,6 +159,10 @@ static bool InitTglShadowWa(struct GfxDeviceInfo *devInfo,
     //source and recon surfaces need to be aligned to the LCU size
     waTable->WaAlignYUVResourceToLCU = 1;
 
+    /* For TGLLP and derivative platforms HW supports both 16K and 64K Aux granularity, POR mode is 64K. On all aux based
+    platforms (where FtrFlaPhysCCS = 0) WaAuxTable64KGranular is expected to be set. */
+    waTable->WaAuxTable64KGranular = 1;
+
     return true;
 }
 
@@ -182,6 +186,7 @@ static struct GfxDeviceInfo tgllpGt1Info = {
     .productFamily = IGFX_TIGERLAKE_LP,
     .displayFamily = IGFX_GEN12_CORE,
     .renderFamily  = IGFX_GEN12_CORE,
+    .mediaFamily   = IGFX_UNKNOWN_CORE,
     .eGTType       = GTTYPE_GT1,
     .L3CacheSizeInKb = 0,
     .L3BankCount   = 4,
@@ -202,6 +207,7 @@ static struct GfxDeviceInfo tgllpGt2Info = {
     .productFamily = IGFX_TIGERLAKE_LP,
     .displayFamily = IGFX_GEN12_CORE,
     .renderFamily  = IGFX_GEN12_CORE,
+    .mediaFamily   = IGFX_UNKNOWN_CORE,
     .eGTType       = GTTYPE_GT2,
     .L3CacheSizeInKb = 0,
     .L3BankCount   = 8,
@@ -223,6 +229,7 @@ static struct GfxDeviceInfo dg1Gt2Info = {
     .productFamily = IGFX_DG1,
     .displayFamily = IGFX_GEN12_CORE,
     .renderFamily  = IGFX_GEN12_CORE,
+    .mediaFamily   = IGFX_UNKNOWN_CORE,
     .eGTType       = GTTYPE_GT2,
     .L3CacheSizeInKb = 0,
     .L3BankCount   = 8,
@@ -249,6 +256,9 @@ static bool dg1Gt2Device4907 = DeviceInfoFactory<GfxDeviceInfo>::
 
 static bool dg1Gt2Device4908 = DeviceInfoFactory<GfxDeviceInfo>::
     RegisterDevice(0x4908, &dg1Gt2Info);
+
+static bool dg1Gt2Device4909 = DeviceInfoFactory<GfxDeviceInfo>::
+    RegisterDevice(0x4909, &dg1Gt2Info);
 #endif
 
 #ifdef IGFX_GEN12_RKL_SUPPORTED
@@ -257,6 +267,7 @@ static struct GfxDeviceInfo rklGt1Info = {
     .productFamily    = IGFX_ROCKETLAKE,
     .displayFamily    = IGFX_GEN12_CORE,
     .renderFamily     = IGFX_GEN12_CORE,
+    .mediaFamily      = IGFX_UNKNOWN_CORE,
     .eGTType          = GTTYPE_GT1,
     .L3CacheSizeInKb  = 0,
     .L3BankCount      = 4,
@@ -277,6 +288,7 @@ static struct GfxDeviceInfo rklGt1fInfo = {
     .productFamily    = IGFX_ROCKETLAKE,
     .displayFamily    = IGFX_GEN12_CORE,
     .renderFamily     = IGFX_GEN12_CORE,
+    .mediaFamily      = IGFX_UNKNOWN_CORE,
     .eGTType          = GTTYPE_GT0_5,
     .L3CacheSizeInKb  = 0,
     .L3BankCount      = 4,
@@ -317,6 +329,7 @@ static struct GfxDeviceInfo adlsGt1Info = {
     .productFamily    = IGFX_ALDERLAKE_S,
     .displayFamily    = IGFX_GEN12_CORE,
     .renderFamily     = IGFX_GEN12_CORE,
+    .mediaFamily      = IGFX_UNKNOWN_CORE,
     .eGTType          = GTTYPE_GT1,
     .L3CacheSizeInKb  = 0,
     .L3BankCount      = 4,
@@ -338,6 +351,7 @@ static struct GfxDeviceInfo adlsGt1fInfo = {
     .productFamily    = IGFX_ALDERLAKE_S,
     .displayFamily    = IGFX_GEN12_CORE,
     .renderFamily     = IGFX_GEN12_CORE,
+    .mediaFamily      = IGFX_UNKNOWN_CORE,
     .eGTType          = GTTYPE_GT0_5,
     .L3CacheSizeInKb  = 0,
     .L3BankCount      = 4,
@@ -366,6 +380,9 @@ static bool adlsGt1Device4682 = DeviceInfoFactory<GfxDeviceInfo>::
 static bool adlsGt1Device4683 = DeviceInfoFactory<GfxDeviceInfo>::
     RegisterDevice(0x4683, &adlsGt1fInfo);
 
+static bool adlsGt1Device4688 = DeviceInfoFactory<GfxDeviceInfo>::
+    RegisterDevice(0x4688, &adlsGt1Info);
+
 static bool adlsGt1Device4690 = DeviceInfoFactory<GfxDeviceInfo>::
     RegisterDevice(0x4690, &adlsGt1Info);
 
@@ -411,6 +428,7 @@ static struct GfxDeviceInfo adlpGt2Info = {
     .productFamily    = IGFX_ALDERLAKE_P,
     .displayFamily    = IGFX_GEN12_CORE,
     .renderFamily     = IGFX_GEN12_CORE,
+    .mediaFamily      = IGFX_UNKNOWN_CORE,
     .eGTType          = GTTYPE_GT2,
     .L3CacheSizeInKb  = 0,
     .L3BankCount      = 8,
@@ -437,17 +455,13 @@ static bool adlpGt2Device46A2 = DeviceInfoFactory<GfxDeviceInfo>::
 
 static bool adlpGt2Device46A3 = DeviceInfoFactory<GfxDeviceInfo>::
     RegisterDevice(0x46A3, &adlpGt2Info);
-    
+
 static bool adlpGt2Device46A6 = DeviceInfoFactory<GfxDeviceInfo>::
     RegisterDevice(0x46A6, &adlpGt2Info);
-    
-static bool adlpGt2Device46A8 = DeviceInfoFactory<GfxDeviceInfo>::
-    RegisterDevice(0x46A8, &adlpGt2Info);
-    
-static bool adlpGt2Device4628 = DeviceInfoFactory<GfxDeviceInfo>::
-    RegisterDevice(0x4628, &adlpGt2Info);
-    
-    
+
+static bool adlpGt2Device4626 = DeviceInfoFactory<GfxDeviceInfo>::
+    RegisterDevice(0x4626, &adlpGt2Info);
+
 static bool adlpGt2Device46B0 = DeviceInfoFactory<GfxDeviceInfo>::
     RegisterDevice(0x46B0, &adlpGt2Info);
 
@@ -459,7 +473,12 @@ static bool adlpGt2Device46B2 = DeviceInfoFactory<GfxDeviceInfo>::
 
 static bool adlpGt2Device46B3 = DeviceInfoFactory<GfxDeviceInfo>::
     RegisterDevice(0x46B3, &adlpGt2Info);
-    
+
+static bool adlpGt2Device46A8 = DeviceInfoFactory<GfxDeviceInfo>::
+    RegisterDevice(0x46A8, &adlpGt2Info);
+
+static bool adlpGt2Device4628 = DeviceInfoFactory<GfxDeviceInfo>::
+    RegisterDevice(0x4628, &adlpGt2Info);
 
 static bool adlpGt2Device46C0 = DeviceInfoFactory<GfxDeviceInfo>::
     RegisterDevice(0x46C0, &adlpGt2Info);
@@ -472,16 +491,30 @@ static bool adlpGt2Device46C2 = DeviceInfoFactory<GfxDeviceInfo>::
 
 static bool adlpGt2Device46C3 = DeviceInfoFactory<GfxDeviceInfo>::
     RegisterDevice(0x46C3, &adlpGt2Info);
-    
+
 static bool adlpGt2Device46AA = DeviceInfoFactory<GfxDeviceInfo>::
     RegisterDevice(0x46AA, &adlpGt2Info);
-    
+
 static bool adlpGt2Device462A = DeviceInfoFactory<GfxDeviceInfo>::
     RegisterDevice(0x462A, &adlpGt2Info);
 
-static bool adlpGt2Device4626 = DeviceInfoFactory<GfxDeviceInfo>::
-    RegisterDevice(0x4626, &adlpGt2Info);
+static bool rplpGt2DeviceA7A0 = DeviceInfoFactory<GfxDeviceInfo>::
+    RegisterDevice(0xA7A0, &adlpGt2Info);
 
+static bool rplpGt2DeviceA720 = DeviceInfoFactory<GfxDeviceInfo>::
+    RegisterDevice(0xA720, &adlpGt2Info);
+
+static bool rplpGt2DeviceA7A8 = DeviceInfoFactory<GfxDeviceInfo>::
+    RegisterDevice(0xA7A8, &adlpGt2Info);
+
+static bool rplpGt2DeviceA7A1 = DeviceInfoFactory<GfxDeviceInfo>::
+    RegisterDevice(0xA7A1, &adlpGt2Info);
+
+static bool rplpGt2DeviceA721 = DeviceInfoFactory<GfxDeviceInfo>::
+    RegisterDevice(0xA721, &adlpGt2Info);
+
+static bool rplpGt2DeviceA7A9 = DeviceInfoFactory<GfxDeviceInfo>::
+    RegisterDevice(0xA7A9, &adlpGt2Info);
 #endif
 
 #ifdef IGFX_GEN12_ADLN_SUPPORTED
@@ -490,6 +523,7 @@ static struct GfxDeviceInfo adlnGt1Info = {
     .productFamily    = IGFX_ALDERLAKE_N,
     .displayFamily    = IGFX_GEN12_CORE,
     .renderFamily     = IGFX_GEN12_CORE,
+    .mediaFamily      = IGFX_UNKNOWN_CORE,
     .eGTType          = GTTYPE_GT1,
     .L3CacheSizeInKb  = 0,
     .L3BankCount      = 8,
