@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2018-2021, Intel Corporation
+* Copyright (c) 2018-2022, Intel Corporation
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
@@ -43,7 +43,7 @@
 namespace decode {
 
 DecodePipeline::DecodePipeline(
-    CodechalHwInterface *hwInterface,
+    CodechalHwInterfaceNext *hwInterface,
     CodechalDebugInterface *debugInterface):
     MediaPipeline(hwInterface ? hwInterface->GetOsInterface() : nullptr)
 {
@@ -51,9 +51,16 @@ DecodePipeline::DecodePipeline(
 
     DECODE_ASSERT(hwInterface != nullptr);
     m_hwInterface = hwInterface;
+    MOS_STATUS m_status = (InitUserSetting(m_userSettingPtr));
 
     m_singleTaskPhaseSupported =
-        ReadUserFeature(__MEDIA_USER_FEATURE_VALUE_DECODE_SINGLE_TASK_PHASE_ENABLE_ID, m_osInterface ? m_osInterface->pOsContext : nullptr).i32Data ? true : false;
+        ReadUserFeature(m_userSettingPtr, "Decode Single Task Phase Enable", MediaUserSetting::Group::Sequence).Get<bool>();
+
+    m_pCodechalOcaDumper = MOS_New(CodechalOcaDumper);
+    if (!m_pCodechalOcaDumper)
+    {
+        MOS_OS_ASSERTMESSAGE("Initialize CodechalOcaDumper failed!");
+    }
 
     CODECHAL_DEBUG_TOOL(
         DECODE_ASSERT(debugInterface != nullptr);
@@ -154,6 +161,16 @@ MOS_STATUS DecodePipeline::Initialize(void *settings)
     DECODE_CHK_NULL(m_hwInterface);
     DECODE_CHK_STATUS(m_hwInterface->Initialize(codecSettings));
 
+    if (m_mediaCopy == nullptr)
+    {
+        m_mediaCopy = m_hwInterface->CreateMediaCopy(m_osInterface);
+    }
+
+#if USE_CODECHAL_DEBUG_TOOL
+    DECODE_CHK_NULL(m_debugInterface);
+    DECODE_CHK_STATUS(m_debugInterface->SetFastDumpConfig(m_mediaCopy));
+#endif
+
     m_mediaContext = MOS_New(MediaContext, scalabilityDecoder, m_hwInterface, m_osInterface);
     DECODE_CHK_NULL(m_mediaContext);
 
@@ -168,7 +185,7 @@ MOS_STATUS DecodePipeline::Initialize(void *settings)
 
     DECODE_CHK_STATUS(CreateStatusReport());
 
-    m_decodecp = Create_DecodeCpInterface(codecSettings, m_hwInterface);
+    m_decodecp = Create_DecodeCpInterface(codecSettings, m_hwInterface->GetCpInterface(), m_hwInterface->GetOsInterface());
     if (m_decodecp)
     {
         m_decodecp->RegisterParams(codecSettings);
@@ -186,6 +203,9 @@ MOS_STATUS DecodePipeline::Uninitialize()
 {
     DECODE_FUNC_CALL();
 
+    // Wait all cmd completion before delete resource.
+    m_osInterface->pfnWaitAllCmdCompletion(m_osInterface);
+
     Delete_DecodeCpInterface(m_decodecp);
     m_decodecp = nullptr;
 
@@ -389,7 +409,7 @@ MOS_STATUS DecodePipeline::DumpOutput(const DecodeStatusReportData& reportData)
 {
     DECODE_FUNC_CALL();
 
-    if (m_debugInterface->DumpIsEnabled(CodechalDbgAttr::attrDecodeOutputSurface))
+    if (m_debugInterface->DumpIsEnabled(CodechalDbgAttr::attrDecodeOutputSurface) || MOS_TraceKeyEnabled(TR_KEY_DECODE_DSTYUV))
     {
         MOS_SURFACE dstSurface;
         MOS_ZeroMemory(&dstSurface, sizeof(dstSurface));
@@ -416,20 +436,21 @@ MOS_STATUS DecodePipeline::DumpOutput(const DecodeStatusReportData& reportData)
 
             if (!Mos_ResourceIsNull(&sfcDstSurface.OsResource))
             {
-                DECODE_CHK_STATUS(m_allocator->GetSurfaceInfo(&sfcDstSurface));
-                DECODE_CHK_STATUS(m_debugInterface->DumpYUVSurface(
-                    &sfcDstSurface, CodechalDbgAttr::attrSfcOutputSurface, "SfcDstSurf"));
-
 #if (_DEBUG || _RELEASE_INTERNAL)
+                DECODE_CHK_STATUS(m_allocator->GetSurfaceInfo(&sfcDstSurface));
                 //rgb format read from reg key
                 uint32_t sfcOutputRgbFormatFlag =
-                    ReadUserFeature(__MEDIA_USER_FEATURE_VALUE_DECODE_SFC_RGBFORMAT_OUTPUT_DEBUG_ID, m_osInterface->pOsContext).u32Data;
-
+                    ReadUserFeature(m_userSettingPtr, "Decode SFC RGB Format Output", MediaUserSetting::Group::Sequence).Get<uint32_t>();
                 if (sfcOutputRgbFormatFlag)
                 {
                     DECODE_CHK_STATUS(m_debugInterface->DumpRgbDataOnYUVSurface(
                         &sfcDstSurface, CodechalDbgAttr::attrSfcOutputSurface, "SfcDstRgbSurf"));
                 }
+                else
+                {
+                    DECODE_CHK_STATUS(m_debugInterface->DumpYUVSurface(
+                        &sfcDstSurface, CodechalDbgAttr::attrSfcOutputSurface, "SfcDstSurf"));
+                }
 #endif
             }
         }
@@ -514,16 +535,19 @@ MOS_STATUS DecodePipeline::TraceDataDumpOutput(const DecodeStatusReportData &rep
             m_tempOutputSurf->UPlaneOffset.iSurfaceOffset,
             m_tempOutputSurf->VPlaneOffset.iSurfaceOffset,
         };
-        MOS_TraceEvent(EVENT_DECODE_DST_DUMPINFO, EVENT_TYPE_INFO, &eventData, sizeof(eventData), NULL, 0); 
+        MOS_TraceEvent(EVENT_DECODE_DUMPINFO_DST, EVENT_TYPE_INFO, &eventData, sizeof(eventData), NULL, 0); 
 
         ResourceAutoLock resLock(m_allocator, &m_tempOutputSurf->OsResource);
         auto             pData = (uint8_t *)resLock.LockResourceForRead();
+        DECODE_CHK_NULL(pData);
 
         MOS_TraceDataDump(
             "Decode_OutputSurf",
             0,
             pData,
             (uint32_t)m_tempOutputSurf->OsResource.pGmmResInfo->GetSizeMainSurface());
+        
+        m_allocator->UnLock(&m_tempOutputSurf->OsResource);
     }
 
     return MOS_STATUS_SUCCESS;
@@ -537,12 +561,17 @@ MOS_STATUS DecodePipeline::TraceDataDump2ndLevelBB(PMHW_BATCH_BUFFER batchBuffer
     batchBuffer->iLastCurrent = batchBuffer->iSize * batchBuffer->count;
     batchBuffer->dwOffset     = 0;
 
-    DECODE_CHK_STATUS(m_osInterface->pfnDumpTraceGpuData(
-        m_osInterface,
+    ResourceAutoLock resLock(m_allocator, &batchBuffer->OsResource);
+    auto             pData = (uint8_t *)resLock.LockResourceForRead();
+    DECODE_CHK_NULL(pData);
+
+    MOS_TraceDataDump(
         "Decode_2ndLevelCmdBB",
-        0,
-        &batchBuffer->OsResource,
-        batchBuffer->iLastCurrent));
+        m_osInterface->pfnGetGpuContext(m_osInterface),
+        pData,
+        batchBuffer->iLastCurrent);
+
+    m_allocator->UnLock(&batchBuffer->OsResource);
 
     return MOS_STATUS_SUCCESS;
 }
@@ -554,8 +583,7 @@ MOS_STATUS DecodePipeline::ReportVdboxIds(const DecodeStatusMfx& status)
     DECODE_FUNC_CALL();
 
     // report the VDBOX IDs to user feature
-    uint32_t vdboxIds = ReadUserFeature(__MEDIA_USER_FEATURE_VALUE_VDBOX_ID_USED, m_osInterface->pOsContext).u32Data;
-
+    uint32_t vdboxIds = ReadUserFeature(m_userSettingPtr, "Used VDBOX ID", MediaUserSetting::Group::Sequence).Get<uint32_t>();
     for (auto i = 0; i < csInstanceIdMax; i++)
     {
         CsEngineId csEngineId;
@@ -660,7 +688,7 @@ MOS_STATUS DecodePipeline::StatusCheck()
 #endif
 
 #if MOS_EVENT_TRACE_DUMP_SUPPORTED
-        if (MOS_GetTraceEventKeyword() & EVENT_DECODE_DSTYUV_KEYWORD)
+        if (MOS_TraceKeyEnabled(TR_KEY_DECODE_DSTYUV_IN_TRACE))
         {
             const DecodeStatusReportData &reportETWData = statusReport->GetReportData(m_statusCheckCount);
             DECODE_CHK_STATUS(TraceDataDumpOutput(reportETWData));
