@@ -48,15 +48,18 @@
 #define __MOS_INTERFACE_H__
 
 #include "mos_defs.h"
+#include "mos_oca_rtlog_mgr_defs.h"
 #include "mos_os.h"
 #include "media_class_trace.h"
 
 class GpuContextSpecificNext;
 struct _MOS_VIRTUALENGINE_SET_PARAMS;
 struct _MOS_VIRTUALENGINE_INIT_PARAMS;
+
 typedef struct _MOS_VIRTUALENGINE_SET_PARAMS  MOS_VIRTUALENGINE_SET_PARAMS, *PMOS_VIRTUALENGINE_SET_PARAMS;
 typedef struct _MOS_VIRTUALENGINE_INIT_PARAMS MOS_VIRTUALENGINE_INIT_PARAMS, *PMOS_VIRTUALENGINE_INIT_PARAMS;
 typedef struct _MOS_CMD_BUF_ATTRI_VE MOS_CMD_BUF_ATTRI_VE, *PMOS_CMD_BUF_ATTRI_VE;
+typedef struct _MHW_VDBOX_GPUNODE_LIMIT *PMHW_VDBOX_GPUNODE_LIMIT;
 class MosInterface
 {
 protected:
@@ -396,20 +399,6 @@ public:
         MOS_STREAM_HANDLE streamState,
         GPU_CONTEXT_HANDLE gpuContext);
 
-    //!
-    //! \brief    Get GPU context pointer
-    //! \details  Get GPU context pointer
-    //! \param    [in] streamState
-    //!           Handle of Os Stream State
-    //! \param    GPU_CONTEXT_HANDLE gpuContextHandle
-    //!           [in] GPU Context Handle
-    //! \return   void *
-    //!           a pointer to a gpu context
-    //!
-    static void *GetGpuContextbyHandle(
-        MOS_STREAM_HANDLE  streamState,
-        GPU_CONTEXT_HANDLE gpuContextHandle);
-
     //! \brief    Sets the object capture flags for Linux OCA dump
     //! \details  Sets the object capture flags for Linux OCA dump
     //!
@@ -831,6 +820,20 @@ public:
     static GMM_RESOURCE_USAGE_TYPE GetGmmResourceUsageType(
         MOS_HW_RESOURCE_DEF mosUsage);
 
+    //!
+    //! \brief    Get MOS_HW_RESOURCE_DEF
+    //! \details  [Resource Interface] Get Mos HW Resource DEF
+    //!           Caller: HAL & MHW
+    //!
+    //! \param    [in] gmmResUsage
+    //!           Gmm Resource usage as index
+    //!
+    //! \return   MOS_HW_RESOURCE_DEF
+    //!           Mos HW resource definition
+    //!
+    static MOS_HW_RESOURCE_DEF GmmToMosResourceUsageType(
+        GMM_RESOURCE_USAGE_TYPE gmmResUsage);
+
     //!
     //! \brief    Get Cache Policy Memory Object
     //! \details  [Resource Interface] Get Cache Policy Memory Object in GMM corresponding to the resource usage
@@ -849,6 +852,21 @@ public:
         GMM_CLIENT_CONTEXT *gmmClientContext,
         MOS_HW_RESOURCE_DEF mosUsage);
 
+    //!
+    //! \brief    Get default Cache Policy Memory Object
+    //! \details  [Resource Interface] Get Cache Policy Memory Object in GMM corresponding to the resource usage
+    //!           Caller: HAL & MHW
+    //!
+    //! \param    [in] gmmClientContext
+    //!           Handle of gmmClientContext
+    //!           If prociding unknown usage, default state will be returned
+    //!
+    //! \return   MEMORY_OBJECT_CONTROL_STATE
+    //!           The cache policy memory object got from MOS interface
+    //!
+    static MEMORY_OBJECT_CONTROL_STATE GetDefaultCachePolicyMemoryObject(
+        GMM_CLIENT_CONTEXT *gmmClientContext);
+
     //!
     //! \brief    Get Cache Policy Memory Object
     //! \details  [Resource Interface] Get Cache Policy Memory Object in GMM corresponding to the resource usage
@@ -1663,6 +1681,7 @@ public:
     //!
     static MosOcaInterface *GetOcaInterface(MOS_STREAM_HANDLE streamState);
 
+    static MosOcaInterface *GetOcaInterface(MOS_CONTEXT_HANDLE mosContextHandle);
     //!
     //! \brief    Maps the specified executable module into the address space of
     //!           the calling process.
@@ -1989,6 +2008,36 @@ public:
     static PMOS_RESOURCE GetMarkerResource(
         MOS_STREAM_HANDLE streamState);
 
+    //!
+    //! \brief    Check if OS resource is nullptr
+    //! \details  Check if OS resource is nullptr
+    //! \param    PMOS_RESOURCE pOsResource
+    //!           [in] Pointer to OS Resource
+    //! \return   int32_t
+    //!           Return true if nullptr, otherwise false
+    //!
+    static bool MosResourceIsNull(PMOS_RESOURCE   resource);
+
+    //!
+    //! \brief    OS reset resource
+    //! \details  Resets the OS resource
+    //! \param    PMOS_RESOURCE pOsResource
+    //!           [in] Pointer to OS Resource
+    //! \return   void
+    //!           Return NONE
+    //!
+    static void MosResetResource(PMOS_RESOURCE   resource);
+
+    //!
+    //! \brief    Get Gmm Resource Info
+    //! \details  Get Gmm Resource Info
+    //! \param    PMOS_RESOURCE resource
+    //!           [in/out] pointer to OS resource
+    //! \return   MOS_STATUS
+    //!           MOS_STATUS_SUCCESS if successful
+    //!
+    static MOS_STATUS GetGmmResourceInfo(PMOS_RESOURCE resource);
+
     //!
     //! \brief    Get plane offset inside surface
     //! \details  Returns the offset
@@ -2014,25 +2063,39 @@ public:
     //!
     static GMM_RESOURCE_FORMAT MosFmtToGmmFmt(MOS_FORMAT format);
 
+    //!
+    //! \brief  Translate GMM_RESOURCE_FORMAT into MOS_FORMAT
+    //!
+    static MOS_FORMAT GmmFmtToMosFmt(GMM_RESOURCE_FORMAT format);
+
     //!
     //! \brief  Translate MOS_FORMAT into MOS_OS_FORMAT
     //!
-    static MOS_OS_FORMAT MosFmtToOsFmt(MOS_FORMAT format);
+    static uint32_t MosFmtToOsFmt(MOS_FORMAT format);
 
     //!
     //! \brief  Translate MOS_OS_FORMT into MOS_FORMAT
     //!
-    static MOS_FORMAT OsFmtToMosFmt(MOS_OS_FORMAT format);
+    static MOS_FORMAT OsFmtToMosFmt(uint32_t format);
 
     //! \brief    Get usersetting instance for each stream
     //! \details  the user setting instance
-    //! \param    MOS_PLANE_OFFSET planeOffset
-    //!           [in] Reference to MOS_PLANE_OFFSET structure
-    //! \return   int - offset of the plane
+    //! \param    MOS_STREAM_HANDLE streamState
+    //!           [in] streamState
+    //! \return   MediaUserSettingSharedPtr - user setting instance
     //!
     static MediaUserSettingSharedPtr MosGetUserSettingInstance(
         MOS_STREAM_HANDLE streamState);
 
+    //! \brief    Get usersetting instance for each stream
+    //! \details  the user setting instance
+    //! \param    PMOS_CONTEXT mosCtx
+    //!           [in] mosCtx
+    //! \return   MediaUserSettingSharedPtr - user setting instance
+    //!
+    static MediaUserSettingSharedPtr MosGetUserSettingInstance(
+        PMOS_CONTEXT mosCtx);
+
     //!
     //! \brief  Translate MOS_OS_FORMT into MOS_FORMAT
     //!
@@ -2043,6 +2106,84 @@ public:
     //!
     static bool IsMismatchOrderProgrammingSupported();
 
+    //!
+    //! \brief  Translate GMM_TILE_TYPE to MOS_TILE_TYPE
+    //!
+    static MOS_TILE_TYPE MapTileType(GMM_RESOURCE_FLAG flags, GMM_TILE_TYPE type);
+
+    //!
+    //! \brief  Check if Multiple Codec Devices is in use
+    //!
+    static bool IsMultipleCodecDevicesInUse(PMOS_INTERFACE osInterface);
+
+    //!
+    //! \brief  get latest virtual node for encoder and decoder
+    //!
+    static MOS_GPU_NODE GetLatestVirtualNode(MOS_STREAM_HANDLE streamState, MOS_COMPONENT component);
+
+    //!
+    //! \brief  set latest virtual node for encoder and decoder
+    //!
+    static void SetLatestVirtualNode(MOS_STREAM_HANDLE streamState, MOS_GPU_NODE node);
+
+    //!
+    //! \brief  get virtual node for each decoder stream
+    //!
+    static MOS_GPU_NODE GetDecoderVirtualNodePerStream(MOS_STREAM_HANDLE streamState);
+
+    //!
+    //! \brief  set virtual node for each decoder stream
+    //!
+    static void SetDecoderVirtualNodePerStream(MOS_STREAM_HANDLE streamState, MOS_GPU_NODE node);
+
+    //! \brief    Wait for the created Batch Buffer completion event
+    //! \details  Wait for the created Batch Buffer completion event, we will be 
+    //!           woken up for every Batch Buffer completion or when TimeOut expires
+    //! \param    MOS_STREAM_HANDLE streamState
+    //!           [in] Pointer to streamState
+    //! \param    GPU_CONTEXT_HANDLE gpuContextHandle
+    //!           [in] gpuContextHandle
+    //! \param    uint32_t uiTimeOut
+    //!           [in] Wait until signaled or TimeOut ms
+    //! \return   MOS_STATUS
+    //!           Return MOS_STATUS_SUCCESS if success else failure reason
+    //!
+    static MOS_STATUS WaitForBBCompleteNotifyEvent(
+        MOS_STREAM_HANDLE       streamState,
+        GPU_CONTEXT_HANDLE      gpuContextHandle,
+        uint32_t                uiTimeOut);
+
+    //!
+    //! \brief    Register GPU Context with KMD BB Complete Event
+    //! \details  Register GPU Context with KMD BB Complete Event
+    //! \param    MOS_STREAM_HANDLE streamState
+    //!           [in] Pointer to streamState
+    //! \param    GPU_CONTEXT_HANDLE gpuContextHandle
+    //!           [in] GPU context handle
+    //! \return   MOS_STATUS
+    //!           MOS_STATUS_SUCCESS if success else fail reason
+    //!
+    static MOS_STATUS RegisterBBCompleteNotifyEvent(
+        MOS_STREAM_HANDLE   streamState,
+        GPU_CONTEXT_HANDLE  gpuContextHandle);
+
+    static void InsertRTLog(
+        MOS_STREAM_HANDLE streamState,
+        MOS_OCA_RTLOG_COMPONENT_TPYE componentType,
+        bool isErr,
+        int32_t id,
+        uint8_t paramCount,
+        const void *param);
+
+    static void GetRtLogResourceInfo(
+        MOS_STREAM_HANDLE streamState,
+        PMOS_RESOURCE &osResource,
+        uint32_t &size);
+
+    static bool IsPooledResource(MOS_STREAM_HANDLE streamState, PMOS_RESOURCE osResource);
+
+    static uint64_t GetResourceHandle(MOS_STREAM_HANDLE streamState, PMOS_RESOURCE osResource);
+
 private:
     //!
     //! \brief    Init per stream parameters
@@ -2120,7 +2261,7 @@ private:
     //!           os device ctx handle
     //! \return   void
     //!
-    static void MosInitOsApiFailSimulateFlag(MOS_CONTEXT_HANDLE mosCtx);
+    static void MosInitOsApiFailSimulateFlag(MediaUserSettingSharedPtr userSettingPtr);
 
     //!
     //! \brief    Deinit OS API fail simulate flags
@@ -2145,4 +2286,7 @@ private:
 MEDIA_CLASS_DEFINE_END(MosInterface)
 };
 
+#define Mos_ResetResource(resource)     MosInterface::MosResetResource(resource)
+#define Mos_ResourceIsNull(resource)    MosInterface::MosResourceIsNull(resource)
+
 #endif  // __MOS_INTERFACE_H__
