@@ -150,10 +150,11 @@ extern const VEBOX_SPATIAL_ATTRIBUTES_CONFIGURATION g_cInit_VEBOX_SPATIAL_ATTRIB
     }
 };
 
-VpResourceManager::VpResourceManager(MOS_INTERFACE &osInterface, VpAllocator &allocator, VphalFeatureReport &reporting, vp::VpPlatformInterface &vpPlatformInterface)
-    : m_osInterface(osInterface), m_allocator(allocator), m_reporting(reporting), m_vpPlatformInterface(vpPlatformInterface)
+VpResourceManager::VpResourceManager(MOS_INTERFACE &osInterface, VpAllocator &allocator, VphalFeatureReport &reporting, vp::VpPlatformInterface &vpPlatformInterface, MediaCopyBaseState *mediaCopy)
+    : m_osInterface(osInterface), m_allocator(allocator), m_reporting(reporting), m_vpPlatformInterface(vpPlatformInterface), m_mediaCopy(mediaCopy)
 {
     InitSurfaceConfigMap();
+    m_userSettingPtr = m_osInterface.pfnGetUserSettingInstance(&m_osInterface);
 }
 
 VpResourceManager::~VpResourceManager()
@@ -175,6 +176,11 @@ VpResourceManager::~VpResourceManager()
         m_allocator.DestroyVpSurface(m_veboxStatisticsSurface);
     }
 
+    if (m_veboxStatisticsSurfacefor1stPassofSfc2Pass)
+    {
+        m_allocator.DestroyVpSurface(m_veboxStatisticsSurfacefor1stPassofSfc2Pass);
+    }
+
     if (m_veboxRgbHistogram)
     {
         m_allocator.DestroyVpSurface(m_veboxRgbHistogram);
@@ -195,11 +201,41 @@ VpResourceManager::~VpResourceManager()
         m_allocator.DestroyVpSurface(m_vebox3DLookUpTables);
     }
 
+    if (m_vebox3DLookUpTables2D)
+    {
+        m_allocator.DestroyVpSurface(m_vebox3DLookUpTables2D);
+    }
+
+    if (m_3DLutKernelCoefSurface)
+    {
+        m_allocator.DestroyVpSurface(m_3DLutKernelCoefSurface);
+    }
+
+    if (m_veboxDnHVSTables)
+    {
+        m_allocator.DestroyVpSurface(m_veboxDnHVSTables);
+    }
+
     if (m_vebox1DLookUpTables)
     {
         m_allocator.DestroyVpSurface(m_vebox1DLookUpTables);
     }
 
+    if (m_temperalInput)
+    {
+        m_allocator.DestroyVpSurface(m_temperalInput);
+    }
+
+    if (m_internalInput)
+    {
+        m_allocator.DestroyVpSurface(m_internalInput);
+    }
+
+    if (m_hdrResourceManager)
+    {
+        MOS_Delete(m_hdrResourceManager);
+    }
+
     while (!m_intermediaSurfaces.empty())
     {
         VP_SURFACE * surf = m_intermediaSurfaces.back();
@@ -213,6 +249,7 @@ VpResourceManager::~VpResourceManager()
     }
 
     m_allocator.DestroyVpSurface(m_cmfcCoeff);
+    m_allocator.DestroyVpSurface(m_decompressionSyncSurface);
 
     m_allocator.CleanRecycler();
 }
@@ -233,6 +270,7 @@ MOS_STATUS VpResourceManager::OnNewFrameProcessStart(SwFilterPipe &pipe)
 {
     VP_FUNC_CALL();
 
+    MT_LOG1(MT_VP_HAL_ONNEWFRAME_PROC_START, MT_NORMAL, MT_FUNC_START, 1);
     VP_SURFACE *inputSurface    = pipe.GetSurface(true, 0);
     VP_SURFACE *outputSurface   = pipe.GetSurface(false, 0);
     SwFilter   *diFilter        = pipe.GetSwFilter(true, 0, FeatureTypeDi);
@@ -559,6 +597,7 @@ MOS_STATUS VpResourceManager::PrepareFcIntermediateSurface(SwFilterPipe &feature
 void VpResourceManager::OnNewFrameProcessEnd()
 {
     VP_FUNC_CALL();
+    MT_LOG1(MT_VP_HAL_ONNEWFRAME_PROC_END, MT_NORMAL, MT_FUNC_END, 1);
     m_allocator.CleanRecycler();
     m_currentPipeIndex = 0;
     CleanTempSurfaces();
@@ -579,6 +618,7 @@ void VpResourceManager::InitSurfaceConfigMap()
     //               |      |      |      |      |      |      |      |                     |                       _currentOutputSurface
     //               |      |      |      |      |      |      |      |                     |                       |                     _pastOutputSurface*/
     //               |      |      |      |      |      |      |      |                     |                       |                     |                 */
+    // sfc Enable
     AddSurfaceConfig(true,  true,  false, false, true,  false, true,  VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_PAST_REF, VEBOX_SURFACE_FRAME1, VEBOX_SURFACE_FRAME0);
     AddSurfaceConfig(true,  true,  true,  false, true,  false, false, VEBOX_SURFACE_FRAME1, VEBOX_SURFACE_NULL,     VEBOX_SURFACE_NULL,   VEBOX_SURFACE_NULL);
     AddSurfaceConfig(true,  true,  false, false, false, false, true,  VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_NULL,     VEBOX_SURFACE_FRAME1, VEBOX_SURFACE_NULL);
@@ -586,13 +626,30 @@ void VpResourceManager::InitSurfaceConfigMap()
     AddSurfaceConfig(true,  true,  false, false, true,  false, false, VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_PAST_REF, VEBOX_SURFACE_FRAME1, VEBOX_SURFACE_FRAME0);
     AddSurfaceConfig(true,  true,  true,  false, false, false, true,  VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_NULL,     VEBOX_SURFACE_FRAME1, VEBOX_SURFACE_NULL);
     AddSurfaceConfig(true,  true,  true,  false, false, false, false, VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_NULL,     VEBOX_SURFACE_FRAME1, VEBOX_SURFACE_NULL);
+    // outOfBound
+    AddSurfaceConfig(true,  true,  false, true,  true,  false, true,  VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_PAST_REF, VEBOX_SURFACE_FRAME1, VEBOX_SURFACE_FRAME0);
+    AddSurfaceConfig(true,  true,  false, true,  true,  false, false, VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_NULL,     VEBOX_SURFACE_FRAME1, VEBOX_SURFACE_NULL);
+
+    // sfc disable
     AddSurfaceConfig(true,  false,  false, false, true,  false, true,  VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_PAST_REF, VEBOX_SURFACE_FRAME1, VEBOX_SURFACE_OUTPUT);
     AddSurfaceConfig(true,  false,  true,  false, true,  false, false, VEBOX_SURFACE_FRAME1, VEBOX_SURFACE_NULL,     VEBOX_SURFACE_NULL,   VEBOX_SURFACE_NULL);
     AddSurfaceConfig(true,  false,  false, false, false, false, true,  VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_NULL,     VEBOX_SURFACE_OUTPUT, VEBOX_SURFACE_NULL);
     AddSurfaceConfig(true,  false,  false, false, false, false, false, VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_NULL,     VEBOX_SURFACE_FRAME1, VEBOX_SURFACE_NULL);
     AddSurfaceConfig(true,  false,  false, false, true,  false, false, VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_PAST_REF, VEBOX_SURFACE_FRAME1, VEBOX_SURFACE_FRAME0);
-    AddSurfaceConfig(true,  false,  true,  false, false, false, true,  VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_NULL,     VEBOX_SURFACE_FRAME1, VEBOX_SURFACE_NULL);
+    AddSurfaceConfig(true,  false,  true,  false, false, false, true,  VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_NULL,     VEBOX_SURFACE_OUTPUT, VEBOX_SURFACE_NULL);
     AddSurfaceConfig(true,  false,  true,  false, false, false, false, VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_NULL,     VEBOX_SURFACE_FRAME1, VEBOX_SURFACE_NULL);
+
+    //30i -> 30p sfc Enable
+    AddSurfaceConfig(false, true,   false, false, false, false, true,  VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_NULL,     VEBOX_SURFACE_FRAME1, VEBOX_SURFACE_NULL);
+    AddSurfaceConfig(false, true,   false, false, true,  false, false, VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_PAST_REF, VEBOX_SURFACE_FRAME1, VEBOX_SURFACE_FRAME0);
+    AddSurfaceConfig(false, true,   false, false, true,  false, true,  VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_PAST_REF, VEBOX_SURFACE_FRAME1, VEBOX_SURFACE_FRAME0);
+    AddSurfaceConfig(false, true,   false, true,  true,  false, true,  VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_PAST_REF, VEBOX_SURFACE_FRAME1, VEBOX_SURFACE_FRAME0);
+    AddSurfaceConfig(false, true,   false, true,  true,  false, false, VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_PAST_REF, VEBOX_SURFACE_FRAME1, VEBOX_SURFACE_FRAME0);
+    
+    //30i -> 30p sfc disable
+    AddSurfaceConfig(false, false,  false, false, true,  false, true,  VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_NULL,     VEBOX_SURFACE_OUTPUT, VEBOX_SURFACE_NULL);
+    AddSurfaceConfig(false, false,  false, true,  true,  false, true,  VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_NULL,     VEBOX_SURFACE_OUTPUT, VEBOX_SURFACE_NULL);
+    AddSurfaceConfig(false, false,  false, false, false, false, true,  VEBOX_SURFACE_INPUT,  VEBOX_SURFACE_NULL,     VEBOX_SURFACE_OUTPUT, VEBOX_SURFACE_NULL);
 }
 
 uint32_t VpResourceManager::GetHistogramSurfaceSize(VP_EXECUTE_CAPS& caps, uint32_t inputWidth, uint32_t inputHeight)
@@ -648,7 +705,7 @@ struct VP_SURFACE_PARAMS
     RECT                    rcMaxSrc            = {0, 0, 0, 0};  //!< Max source rectangle
     VPHAL_SAMPLE_TYPE       sampleType          = SAMPLE_PROGRESSIVE;
 };
-MOS_STATUS VpResourceManager::Get3DLutOutputColorAndFormat(VPHAL_CSPACE &colorSpace, MOS_FORMAT &format, SwFilterPipe &executedFilters)
+MOS_STATUS VpResourceManager::GetIntermediaColorAndFormat3DLutOutput(VPHAL_CSPACE &colorSpace, MOS_FORMAT &format, SwFilterPipe &executedFilters)
 {
     SwFilterHdr *hdr = dynamic_cast<SwFilterHdr *>(executedFilters.GetSwFilter(true, 0, FeatureType::FeatureTypeHdr));
     if (hdr)
@@ -664,6 +721,33 @@ MOS_STATUS VpResourceManager::Get3DLutOutputColorAndFormat(VPHAL_CSPACE &colorSp
     }
     return MOS_STATUS_SUCCESS;
 }
+
+MOS_STATUS VpResourceManager::GetIntermediaColorAndFormatBT2020toRGB(VP_EXECUTE_CAPS &caps, VPHAL_CSPACE &colorSpace, MOS_FORMAT &format, SwFilterPipe &executedFilters)
+{
+    SwFilterCsc *cscOnSfc = dynamic_cast<SwFilterCsc *>(executedFilters.GetSwFilter(true, 0, FeatureType::FeatureTypeCscOnSfc));
+    SwFilterCgc *cgc      = dynamic_cast<SwFilterCgc *>(executedFilters.GetSwFilter(true, 0, FeatureType::FeatureTypeCgc));
+
+    if (caps.bSFC && nullptr == cscOnSfc)
+    {
+        VP_PUBLIC_CHK_STATUS_RETURN(MOS_STATUS_INVALID_PARAMETER);
+    }
+
+    if (cscOnSfc)
+    {
+        colorSpace = cscOnSfc->GetSwFilterParams().output.colorSpace;
+        format     = cscOnSfc->GetSwFilterParams().formatOutput;
+    }
+    else
+    {
+        VP_PUBLIC_CHK_NULL_RETURN(cgc);
+        colorSpace = cgc->GetSwFilterParams().dstColorSpace;
+        format     = cgc->GetSwFilterParams().formatOutput;
+    }
+
+    return MOS_STATUS_SUCCESS;
+}
+
+
 MOS_STATUS VpResourceManager::GetIntermediaOutputSurfaceColorAndFormat(VP_EXECUTE_CAPS &caps, SwFilterPipe &executedFilters, MOS_FORMAT &format, VPHAL_CSPACE &colorSpace)
 {
     VP_SURFACE *inputSurface = executedFilters.GetSurface(true, 0);
@@ -691,7 +775,12 @@ MOS_STATUS VpResourceManager::GetIntermediaOutputSurfaceColorAndFormat(VP_EXECUT
     }
     else if (caps.b3DlutOutput)
     {
-        VP_PUBLIC_CHK_STATUS_RETURN(Get3DLutOutputColorAndFormat(colorSpace, format, executedFilters));
+        VP_PUBLIC_CHK_STATUS_RETURN(GetIntermediaColorAndFormat3DLutOutput(colorSpace, format, executedFilters));
+        return MOS_STATUS_SUCCESS;
+    }
+    else if (caps.bBt2020ToRGB)
+    {
+        VP_PUBLIC_CHK_STATUS_RETURN(GetIntermediaColorAndFormatBT2020toRGB(caps, colorSpace, format, executedFilters));
         return MOS_STATUS_SUCCESS;
     }
     else if (caps.bVebox)
@@ -760,8 +849,19 @@ MOS_STATUS VpResourceManager::GetIntermediaOutputSurfaceParams(VP_EXECUTE_CAPS&
     VP_PUBLIC_CHK_STATUS_RETURN(GetIntermediaOutputSurfaceColorAndFormat(caps, executedFilters, params.format, params.colorSpace));
 
     params.tileType = MOS_TILE_Y;
-    params.surfCompressionMode = caps.bRender ? MOS_MMC_RC : MOS_MMC_MC;
-    params.surfCompressible = true;
+
+    if (SAMPLE_PROGRESSIVE == params.sampleType)
+    {
+        params.surfCompressionMode = caps.bRender ? MOS_MMC_RC : MOS_MMC_MC;
+        params.surfCompressible    = true;
+    }
+    else
+    {
+        // MMC does not support interleaved surface.
+        VP_PUBLIC_NORMALMESSAGE("Disable MMC for interleaved intermedia surface, sampleType = %d.", params.sampleType);
+        params.surfCompressionMode = MOS_MMC_DISABLED;
+        params.surfCompressible    = false;
+    }
 
     return MOS_STATUS_SUCCESS;
 }
@@ -914,27 +1014,46 @@ MOS_STATUS VpResourceManager::AssignFcResources(VP_EXECUTE_CAPS &caps, std::vect
         memTypeSurfVideoMem = MOS_MEMPOOL_DEVICEMEMORY;
     }
 
-    for (size_t i = 0; i < inputSurfaces.size(); ++i)
+    if (caps.bTemperalInputInuse)
     {
-        surfSetting.surfGroup.insert(std::make_pair((SurfaceType)(SurfaceTypeFcInputLayer0 + i), inputSurfaces[i]));
-
-        if (!resHint.isIScalingTypeNone)
+        if (inputSurfaces.size() > 1)
         {
-            // For Interlaced scaling, 2nd field is part of the same frame.
-            // For Field weaving, 2nd field is passed in as a ref.
-            VP_SURFACE *surfField1Dual = nullptr;
-            if (resHint.isFieldWeaving)
+            VP_PUBLIC_ASSERTMESSAGE("Temperal input only has 1 layer, do not support multi-layer case!");
+            return MOS_STATUS_INVALID_PARAMETER;
+        }
+        surfSetting.surfGroup.insert(std::make_pair((SurfaceType)(SurfaceTypeFcInputLayer0), m_temperalInput));
+    }
+    else
+    {
+        for (size_t i = 0; i < inputSurfaces.size(); ++i)
+        {
+            if (caps.bInternalInputInuse && i==0)
             {
-                surfField1Dual = pastSurfaces[i];
-                VP_PUBLIC_NORMALMESSAGE("Field weaving case. 2nd field is passed in as a ref.");
+                surfSetting.surfGroup.insert(std::make_pair((SurfaceType)(SurfaceTypeFcInputLayer0 + i), m_internalInput));
             }
             else
             {
-                surfField1Dual = GetCopyInstOfExtSurface(inputSurfaces[i]);
-                VP_PUBLIC_NORMALMESSAGE("Interlaced scaling. 2nd field is part of the same frame.");
+                surfSetting.surfGroup.insert(std::make_pair((SurfaceType)(SurfaceTypeFcInputLayer0 + i), inputSurfaces[i]));
+            }
+
+            if (!resHint.isIScalingTypeNone)
+            {
+                // For Interlaced scaling, 2nd field is part of the same frame.
+                // For Field weaving, 2nd field is passed in as a ref.
+                VP_SURFACE *surfField1Dual = nullptr;
+                if (resHint.isFieldWeaving)
+                {
+                    surfField1Dual = pastSurfaces[i];
+                    VP_PUBLIC_NORMALMESSAGE("Field weaving case. 2nd field is passed in as a ref.");
+                }
+                else
+                {
+                    surfField1Dual = GetCopyInstOfExtSurface(inputSurfaces[i]);
+                    VP_PUBLIC_NORMALMESSAGE("Interlaced scaling. 2nd field is part of the same frame.");
+                }
+                VP_PUBLIC_CHK_NULL_RETURN(surfField1Dual);
+                surfSetting.surfGroup.insert(std::make_pair((SurfaceType)(SurfaceTypeFcInputLayer0Field1Dual + i), surfField1Dual));
             }
-            VP_PUBLIC_CHK_NULL_RETURN(surfField1Dual);
-            surfSetting.surfGroup.insert(std::make_pair((SurfaceType)(SurfaceTypeFcInputLayer0Field1Dual + i), surfField1Dual));
         }
     }
     surfSetting.surfGroup.insert(std::make_pair(SurfaceTypeFcTarget0, outputSurface));
@@ -956,21 +1075,52 @@ MOS_STATUS VpResourceManager::AssignFcResources(VP_EXECUTE_CAPS &caps, std::vect
         MOS_HW_RESOURCE_USAGE_VP_INTERNAL_READ_RENDER,
         MOS_TILE_UNSET_GMM,
         memTypeSurfVideoMem,
-        MOS_MEMPOOL_DEVICEMEMORY == memTypeSurfVideoMem));
+        VPP_INTER_RESOURCE_NOTLOCKABLE));
 
     surfSetting.surfGroup.insert(std::make_pair(SurfaceTypeFcCscCoeff, m_cmfcCoeff));
 
+    //for decompreesion sync on interlace input of FC
+    VP_PUBLIC_CHK_STATUS_RETURN(m_allocator.ReAllocateSurface(
+        m_decompressionSyncSurface,
+        "AuxDecompressSyncSurface",
+        Format_Buffer,
+        MOS_GFXRES_BUFFER,
+        MOS_TILE_LINEAR,
+        32,
+        1,
+        false,
+        MOS_MMC_DISABLED,
+        allocated));
+    surfSetting.surfGroup.insert(std::make_pair(SurfaceTypeDecompressionSync, m_decompressionSyncSurface));
+
     return MOS_STATUS_SUCCESS;
 }
 
 MOS_STATUS VpResourceManager::AssignRenderResource(VP_EXECUTE_CAPS &caps, std::vector<VP_SURFACE *> &inputSurfaces, VP_SURFACE *outputSurface,
-    std::vector<VP_SURFACE *> &pastSurfaces, std::vector<VP_SURFACE *> &futureSurfaces, RESOURCE_ASSIGNMENT_HINT resHint, VP_SURFACE_SETTING &surfSetting)
+    std::vector<VP_SURFACE *> &pastSurfaces, std::vector<VP_SURFACE *> &futureSurfaces, RESOURCE_ASSIGNMENT_HINT resHint, VP_SURFACE_SETTING &surfSetting, SwFilterPipe& executedFilters)
 {
     VP_FUNC_CALL();
 
     if (caps.bComposite)
     {
         VP_PUBLIC_CHK_STATUS_RETURN(AssignFcResources(caps, inputSurfaces, outputSurface, pastSurfaces, futureSurfaces, resHint, surfSetting));
+        if (caps.bInternalInputInuse)
+        {
+            m_internalInput->ColorSpace = inputSurfaces[0]->ColorSpace;
+            executedFilters.AddSurface(m_internalInput, true, 0);
+        }
+    }
+    else if (caps.b3DLutCalc)
+    {
+        VP_PUBLIC_CHK_STATUS_RETURN(Assign3DLutKernelResource(caps, resHint, surfSetting));
+    }
+    else if (caps.bHVSCalc)
+    {
+        VP_PUBLIC_CHK_STATUS_RETURN(AssignHVSKernelResource(caps, resHint, surfSetting));
+    }
+    else if (caps.bRenderHdr)
+    {
+        VP_PUBLIC_CHK_STATUS_RETURN(AssignHdrResource(caps, inputSurfaces, outputSurface, resHint, surfSetting, executedFilters));
     }
     else
     {
@@ -1027,13 +1177,43 @@ MOS_STATUS VpResourceManager::AssignExecuteResource(std::vector<FeatureType> &fe
     }
 
     VP_PUBLIC_CHK_STATUS_RETURN(AssignExecuteResource(caps, inputSurfaces, outputSurface,
-        pastSurfaces, futureSurfaces, resHint, executedFilters.GetSurfacesSetting()));
+        pastSurfaces, futureSurfaces, resHint, executedFilters.GetSurfacesSetting(), executedFilters));
     ++m_currentPipeIndex;
     return MOS_STATUS_SUCCESS;
 }
 
+MOS_STATUS VpResourceManager::GetUpdatedExecuteResource(std::vector<FeatureType> &featurePool, VP_EXECUTE_CAPS &caps, SwFilterPipe &swfilterPipe, VP_SURFACE_SETTING &surfSetting)
+{
+    VP_FUNC_CALL();
+
+    std::vector<VP_SURFACE *> inputSurfaces, pastSurfaces, futureSurfaces;
+    for (uint32_t i = 0; i < swfilterPipe.GetSurfaceCount(true); ++i)
+    {
+        VP_SURFACE *inputSurface = GetCopyInstOfExtSurface(swfilterPipe.GetSurface(true, i));
+        VP_PUBLIC_CHK_NULL_RETURN(inputSurface);
+        inputSurfaces.push_back(inputSurface);
+
+        VP_SURFACE *pastSurface = GetCopyInstOfExtSurface(swfilterPipe.GetPastSurface(i));
+        pastSurfaces.push_back(pastSurface ? pastSurface : nullptr);
+
+        VP_SURFACE *futureSurface = GetCopyInstOfExtSurface(swfilterPipe.GetFutureSurface(i));
+        futureSurfaces.push_back(futureSurface ? futureSurface : nullptr);
+    }
+    VP_SURFACE *outputSurface  = GetCopyInstOfExtSurface(swfilterPipe.GetSurface(false, 0));
+
+    RESOURCE_ASSIGNMENT_HINT resHint = {};
+
+    VP_PUBLIC_CHK_STATUS_RETURN(GetResourceHint(featurePool, swfilterPipe, resHint));
+
+    VP_PUBLIC_CHK_STATUS_RETURN(AssignExecuteResource(caps, inputSurfaces, outputSurface,
+        pastSurfaces, futureSurfaces, resHint, surfSetting, swfilterPipe));
+    ++m_currentPipeIndex;
+
+    return MOS_STATUS_SUCCESS;
+}
+
 MOS_STATUS VpResourceManager::AssignExecuteResource(VP_EXECUTE_CAPS& caps, std::vector<VP_SURFACE *> &inputSurfaces, VP_SURFACE *outputSurface,
-    std::vector<VP_SURFACE *> &pastSurfaces, std::vector<VP_SURFACE *> &futureSurfaces, RESOURCE_ASSIGNMENT_HINT resHint, VP_SURFACE_SETTING &surfSetting)
+    std::vector<VP_SURFACE *> &pastSurfaces, std::vector<VP_SURFACE *> &futureSurfaces, RESOURCE_ASSIGNMENT_HINT resHint, VP_SURFACE_SETTING &surfSetting, SwFilterPipe& executedFilters)
 {
     VP_FUNC_CALL();
 
@@ -1042,12 +1222,12 @@ MOS_STATUS VpResourceManager::AssignExecuteResource(VP_EXECUTE_CAPS& caps, std::
     if (caps.bVebox || caps.bDnKernelUpdate)
     {
         // Create Vebox Resources
-        VP_PUBLIC_CHK_STATUS_RETURN(AssignVeboxResource(caps, inputSurfaces[0], outputSurface, pastSurfaces[0], futureSurfaces[0], resHint, surfSetting));
+        VP_PUBLIC_CHK_STATUS_RETURN(AssignVeboxResource(caps, inputSurfaces[0], outputSurface, pastSurfaces[0], futureSurfaces[0], resHint, surfSetting, executedFilters));
     }
 
     if (caps.bRender)
     {
-        VP_PUBLIC_CHK_STATUS_RETURN(AssignRenderResource(caps, inputSurfaces, outputSurface, pastSurfaces, futureSurfaces, resHint, surfSetting));
+        VP_PUBLIC_CHK_STATUS_RETURN(AssignRenderResource(caps, inputSurfaces, outputSurface, pastSurfaces, futureSurfaces, resHint, surfSetting, executedFilters));
     }
 
     return MOS_STATUS_SUCCESS;
@@ -1193,7 +1373,7 @@ MOS_STATUS VpResourceManager::ReAllocateVeboxOutputSurface(VP_EXECUTE_CAPS& caps
             MOS_HW_RESOURCE_USAGE_VP_OUTPUT_PICTURE_FF,
             MOS_TILE_UNSET_GMM,
             memTypeSurfVideoMem,
-            MOS_MEMPOOL_DEVICEMEMORY == memTypeSurfVideoMem));
+            VPP_INTER_RESOURCE_NOTLOCKABLE));
 
         m_veboxOutput[i]->ColorSpace = inputSurface->ColorSpace;
         m_veboxOutput[i]->rcDst      = inputSurface->rcDst;
@@ -1234,7 +1414,7 @@ MOS_STATUS VpResourceManager::ReAllocateVeboxDenoiseOutputSurface(VP_EXECUTE_CAP
     if (skuTable)
     {
         //DN output surface must be tile64 only when input format is bayer
-        if (MEDIA_IS_SKU(skuTable, FtrMediaTile64) &&
+        if (!MEDIA_IS_SKU(skuTable, FtrTileY) &&
             IS_BAYER_FORMAT(inputSurface->osSurface->Format))
         {
             tileModeByForce = MOS_TILE_64_GMM;
@@ -1299,7 +1479,7 @@ MOS_STATUS VpResourceManager::ReAllocateVeboxDenoiseOutputSurface(VP_EXECUTE_CAP
             MOS_HW_RESOURCE_USAGE_VP_INPUT_REFERENCE_FF,
             tileModeByForce,
             memTypeSurfVideoMem,
-            MOS_MEMPOOL_DEVICEMEMORY == memTypeSurfVideoMem));
+            VPP_INTER_RESOURCE_NOTLOCKABLE));
 
         // if allocated, pVeboxState->PastSurface is not valid for DN reference.
         if (allocated)
@@ -1428,13 +1608,16 @@ MOS_STATUS VpResourceManager::ReAllocateVeboxSTMMSurface(VP_EXECUTE_CAPS& caps,
     uint32_t                        i                   = 0;
     MOS_TILE_MODE_GMM               tileModeByForce     = MOS_TILE_UNSET_GMM;
     auto *                          skuTable            = MosInterface::GetSkuTable(m_osInterface.osStreamState);
-    Mos_MemPool                     memTypeHistStat     = GetHistStatMemType();
+    Mos_MemPool                     memTypeHistStat     = GetHistStatMemType(caps);
     uint32_t                        dwHeight;
 
+    //STMM surface can be not lockable, if secure mode is enabled
+    bool isSTMMNotLockable = caps.bSecureVebox;
+
     VP_PUBLIC_CHK_NULL_RETURN(inputSurface);
     VP_PUBLIC_CHK_NULL_RETURN(inputSurface->osSurface);
 
-    if (skuTable && MEDIA_IS_SKU(skuTable, FtrMediaTile64))
+    if (skuTable && !MEDIA_IS_SKU(skuTable, FtrTileY))
     {
         tileModeByForce = MOS_TILE_64_GMM;
     }
@@ -1476,12 +1659,12 @@ MOS_STATUS VpResourceManager::ReAllocateVeboxSTMMSurface(VP_EXECUTE_CAPS& caps,
             MOS_HW_RESOURCE_USAGE_VP_INTERNAL_READ_WRITE_FF,
             tileModeByForce,
             memTypeHistStat,
-            MOS_MEMPOOL_DEVICEMEMORY == memTypeHistStat));
+            isSTMMNotLockable));
 
         if (allocated)
         {
             VP_PUBLIC_CHK_NULL_RETURN(m_veboxSTMMSurface[i]);
-            if (MOS_MEMPOOL_DEVICEMEMORY != memTypeHistStat)
+            if (!isSTMMNotLockable)
             {
                 VP_PUBLIC_CHK_STATUS_RETURN(VeboxInitSTMMHistory(m_veboxSTMMSurface[i]->osSurface));
             }
@@ -1524,10 +1707,20 @@ void VpResourceManager::DestoryVeboxSTMMSurface()
     }
 }
 
-uint32_t VpResourceManager::Get3DLutSize()
+MOS_STATUS VpResourceManager::FillLinearBufferWithEncZero(VP_SURFACE *surface, uint32_t width, uint32_t height)
 {
     VP_FUNC_CALL();
 
+    return MOS_STATUS_SUCCESS;
+}
+
+uint32_t VpResourceManager::Get3DLutSize(uint32_t &lutWidth, uint32_t &lutHeight)
+{
+    VP_FUNC_CALL();
+
+    lutWidth = LUT65_SEG_SIZE * 2;
+    lutHeight = LUT65_SEG_SIZE * LUT65_MUL_SIZE;
+
     return VP_VEBOX_HDR_3DLUT65;
 }
 
@@ -1538,7 +1731,7 @@ uint32_t VpResourceManager::Get1DLutSize()
     return SHAPE_1K_LOOKUP_SIZE;
 }
 
-Mos_MemPool VpResourceManager::GetHistStatMemType()
+Mos_MemPool VpResourceManager::GetHistStatMemType(VP_EXECUTE_CAPS &caps)
 {
     VP_FUNC_CALL();
 
@@ -1554,11 +1747,12 @@ MOS_STATUS VpResourceManager::AllocateVeboxResource(VP_EXECUTE_CAPS& caps, VP_SU
     uint32_t                        dwHeight;
     uint32_t                        dwSize;
     uint32_t                        i;
-    MOS_RESOURCE_MMC_MODE           surfCompressionMode = MOS_MMC_DISABLED;
-    bool                            bSurfCompressible   = false;
-    bool                            bAllocated          = false;
-    uint8_t                         InitValue           = 0;
-    Mos_MemPool                     memTypeHistStat     = GetHistStatMemType();
+    MOS_RESOURCE_MMC_MODE           surfCompressionMode        = MOS_MMC_DISABLED;
+    bool                            bSurfCompressible          = false;
+    bool                            bAllocated                 = false;
+    uint8_t                         InitValue                  = 0;
+    Mos_MemPool                     memTypeHistStat            = GetHistStatMemType(caps);
+    bool                            isStatisticsBufNotLockable = false;
 
     VP_PUBLIC_CHK_NULL_RETURN(inputSurface);
     VP_PUBLIC_CHK_NULL_RETURN(inputSurface->osSurface);
@@ -1566,19 +1760,19 @@ MOS_STATUS VpResourceManager::AllocateVeboxResource(VP_EXECUTE_CAPS& caps, VP_SU
     VP_PUBLIC_CHK_NULL_RETURN(outputSurface->osSurface);
 
     // change the init value when null hw is enabled
-    if (NullHW::IsEnabled())
+    if (m_osInterface.bNullHwIsEnabled)
     {
         InitValue = 0x80;
     }
 
     if (IS_VP_VEBOX_DN_ONLY(caps))
     {
-        bSurfCompressible = inputSurface->osSurface->bCompressible;
+        bSurfCompressible   = inputSurface->osSurface->bCompressible;
         surfCompressionMode = inputSurface->osSurface->CompressionMode;
     }
     else
     {
-        bSurfCompressible = true;
+        bSurfCompressible   = true;
         surfCompressionMode = MOS_MMC_MC;
     }
 
@@ -1597,7 +1791,7 @@ MOS_STATUS VpResourceManager::AllocateVeboxResource(VP_EXECUTE_CAPS& caps, VP_SU
         VP_PUBLIC_CHK_STATUS_RETURN(ReAllocateVeboxDenoiseOutputSurface(caps, inputSurface, bAllocated));
         if (bAllocated)
         {
-            m_currentDnOutput = 0;
+            m_currentDnOutput   = 0;
             m_pastDnOutputValid = false;
         }
     }
@@ -1679,14 +1873,11 @@ MOS_STATUS VpResourceManager::AllocateVeboxResource(VP_EXECUTE_CAPS& caps, VP_SU
         bAllocated,
         false,
         IsDeferredResourceDestroyNeeded(),
-        MOS_HW_RESOURCE_USAGE_VP_INTERNAL_WRITE_FF,
-        MOS_TILE_UNSET_GMM,
-        memTypeHistStat,
-        MOS_MEMPOOL_DEVICEMEMORY == memTypeHistStat));
+        MOS_HW_RESOURCE_USAGE_VP_INTERNAL_WRITE_FF));
 
     m_isHistogramReallocated = bAllocated;
 
-    if (bAllocated && NullHW::IsEnabled())
+    if (bAllocated && m_osInterface.bNullHwIsEnabled)
     {
         // Initialize veboxRgbHistogram Surface
         VP_PUBLIC_CHK_STATUS_RETURN(m_allocator.OsFillResource(
@@ -1701,61 +1892,20 @@ MOS_STATUS VpResourceManager::AllocateVeboxResource(VP_EXECUTE_CAPS& caps, VP_SU
     // Surface to be a rectangle aligned with dwWidth to get proper dwSize
     // APG PAth need to make sure input surface width/height is what to processed width/Height
     uint32_t statistic_size = m_vpPlatformInterface.VeboxQueryStaticSurfaceSize();
-    dwWidth = MOS_ALIGN_CEIL(inputSurface->osSurface->dwWidth, 64);
-    dwHeight = MOS_ROUNDUP_DIVIDE(inputSurface->osSurface->dwHeight, 4) +
-               MOS_ROUNDUP_DIVIDE(statistic_size * sizeof(uint32_t), dwWidth);
-    dwSize = dwWidth * dwHeight;
+    dwWidth                 = MOS_ALIGN_CEIL(inputSurface->osSurface->dwWidth, 64);
+    dwHeight                = MOS_ROUNDUP_DIVIDE(inputSurface->osSurface->dwHeight, 4) +
+                MOS_ROUNDUP_DIVIDE(statistic_size * sizeof(uint32_t), dwWidth);
 
-    VP_PUBLIC_CHK_STATUS_RETURN(m_allocator.ReAllocateSurface(
-        m_veboxStatisticsSurface,
-        "VeboxStatisticsSurface",
-        Format_Buffer,
-        MOS_GFXRES_BUFFER,
-        MOS_TILE_LINEAR,
-        dwWidth,
-        dwHeight,
-        false,
-        MOS_MMC_DISABLED,
-        bAllocated,
-        true,
-        IsDeferredResourceDestroyNeeded(),
-        MOS_HW_RESOURCE_USAGE_VP_INTERNAL_WRITE_FF,
-        MOS_TILE_UNSET_GMM,
-        memTypeHistStat,
-        MOS_MEMPOOL_DEVICEMEMORY == memTypeHistStat));
-
-    if (bAllocated)
+    if (caps.b1stPassOfSfc2PassScaling)
     {
-        if (MOS_MEMPOOL_DEVICEMEMORY != memTypeHistStat)
-        {
-            // Initialize veboxStatisticsSurface Surface
-            VP_PUBLIC_CHK_STATUS_RETURN(m_allocator.OsFillResource(
-                &(m_veboxStatisticsSurface->osSurface->OsResource),
-                dwSize,
-                InitValue));
-            m_dwVeboxPerBlockStatisticsWidth  = dwWidth;
-            m_dwVeboxPerBlockStatisticsHeight = MOS_ROUNDUP_DIVIDE(inputSurface->osSurface->dwHeight, 4);
-        }
+        VP_PUBLIC_CHK_STATUS_RETURN(ReAllocateVeboxStatisticsSurface(m_veboxStatisticsSurfacefor1stPassofSfc2Pass, caps, inputSurface, dwWidth, dwHeight));
+    }
+    else
+    {
+        VP_PUBLIC_CHK_STATUS_RETURN(ReAllocateVeboxStatisticsSurface(m_veboxStatisticsSurface, caps, inputSurface, dwWidth, dwHeight));
     }
 
-    if (caps.bHDR3DLUT)
-    {
-        // HDR
-        dwSize = Get3DLutSize();
-        VP_PUBLIC_CHK_STATUS_RETURN(m_allocator.ReAllocateSurface(
-            m_vebox3DLookUpTables,
-            "Vebox3DLutTableSurface",
-            Format_Buffer,
-            MOS_GFXRES_BUFFER,
-            MOS_TILE_LINEAR,
-            dwSize,
-            1,
-            false,
-            MOS_MMC_DISABLED,
-            bAllocated,
-            false,
-            IsDeferredResourceDestroyNeeded()));
-    }
+    VP_PUBLIC_CHK_STATUS_RETURN(Allocate3DLut(caps));
 
     if (caps.bDV)
     {
@@ -1779,6 +1929,123 @@ MOS_STATUS VpResourceManager::AllocateVeboxResource(VP_EXECUTE_CAPS& caps, VP_SU
     return MOS_STATUS_SUCCESS;
 }
 
+MOS_STATUS VpResourceManager::Allocate3DLut(VP_EXECUTE_CAPS& caps)
+{
+    VP_FUNC_CALL();
+    uint32_t                        size = 0;
+    bool                            isAllocated          = false;
+
+    if (caps.bHDR3DLUT || caps.b3DLutCalc)
+    {
+        // HDR
+        uint32_t lutWidth = 0;
+        uint32_t lutHeight = 0;
+        size = Get3DLutSize(lutWidth, lutHeight);
+        VP_PUBLIC_CHK_STATUS_RETURN(m_allocator.ReAllocateSurface(
+            m_vebox3DLookUpTables,
+            "Vebox3DLutTableSurface",
+            Format_Buffer,
+            MOS_GFXRES_BUFFER,
+            MOS_TILE_LINEAR,
+            size,
+            1,
+            false,
+            MOS_MMC_DISABLED,
+            isAllocated,
+            false,
+            IsDeferredResourceDestroyNeeded(),
+            MOS_HW_RESOURCE_USAGE_VP_INTERNAL_READ_WRITE_RENDER));
+
+    }
+
+    return MOS_STATUS_SUCCESS;
+}
+
+MOS_STATUS VpResourceManager::AllocateResourceFor3DLutKernel(VP_EXECUTE_CAPS& caps)
+{
+    VP_FUNC_CALL();
+    uint32_t    size = 0;
+    bool        isAllocated = false;
+    uint32_t    lutWidth = 0;
+    uint32_t    lutHeight = 0;
+
+    uint32_t sizeOf3DLut = Get3DLutSize(lutWidth, lutHeight);
+    if (VP_VEBOX_HDR_3DLUT65 != sizeOf3DLut)
+    {
+        VP_PUBLIC_ASSERTMESSAGE("3DLutSize(%x) != VP_VEBOX_HDR_3DLUT65(%x)", sizeOf3DLut, VP_VEBOX_HDR_3DLUT65);
+        VP_PUBLIC_CHK_STATUS_RETURN(MOS_STATUS_INVALID_PARAMETER);
+    }
+
+    VP_PUBLIC_CHK_STATUS_RETURN(Allocate3DLut(caps));
+
+    uint32_t size_coef     = 8 * 8 * 4;
+
+    VP_PUBLIC_CHK_STATUS_RETURN(m_allocator.ReAllocateSurface(
+        m_3DLutKernelCoefSurface,
+        "3DLutKernelCoefSurface",
+        Format_Buffer,
+        MOS_GFXRES_BUFFER,
+        MOS_TILE_LINEAR,
+        size_coef,
+        1,
+        false,
+        MOS_MMC_DISABLED,
+        isAllocated,
+        false,
+        IsDeferredResourceDestroyNeeded(),
+        MOS_HW_RESOURCE_USAGE_VP_INTERNAL_READ_WRITE_RENDER));
+
+    return MOS_STATUS_SUCCESS;
+}
+
+MOS_STATUS VpResourceManager::Assign3DLutKernelResource(VP_EXECUTE_CAPS &caps, RESOURCE_ASSIGNMENT_HINT resHint, VP_SURFACE_SETTING &surfSetting)
+{
+    VP_FUNC_CALL();
+
+    VP_PUBLIC_CHK_STATUS_RETURN(AllocateResourceFor3DLutKernel(caps));
+
+    surfSetting.surfGroup.insert(std::make_pair(SurfaceType3DLut, m_vebox3DLookUpTables));
+    surfSetting.surfGroup.insert(std::make_pair(SurfaceType3DLutCoef, m_3DLutKernelCoefSurface));
+
+    return MOS_STATUS_SUCCESS;
+}
+
+MOS_STATUS VpResourceManager::AllocateResourceForHVSKernel(VP_EXECUTE_CAPS &caps)
+{
+    VP_FUNC_CALL();
+    bool     isAllocated = false;
+
+    uint32_t size  = 40 * 4;
+
+    VP_PUBLIC_CHK_STATUS_RETURN(m_allocator.ReAllocateSurface(
+        m_veboxDnHVSTables,
+        "HVSKernelTableSurface",
+        Format_Buffer,
+        MOS_GFXRES_BUFFER,
+        MOS_TILE_LINEAR,
+        size,
+        1,
+        false,
+        MOS_MMC_DISABLED,
+        isAllocated,
+        false,
+        IsDeferredResourceDestroyNeeded(),
+        MOS_HW_RESOURCE_USAGE_VP_INTERNAL_READ_WRITE_RENDER));
+
+    return MOS_STATUS_SUCCESS;
+}
+
+MOS_STATUS VpResourceManager::AssignHVSKernelResource(VP_EXECUTE_CAPS &caps, RESOURCE_ASSIGNMENT_HINT resHint, VP_SURFACE_SETTING &surfSetting)
+{
+    VP_FUNC_CALL();
+
+    VP_PUBLIC_CHK_STATUS_RETURN(AllocateResourceForHVSKernel(caps));
+
+    surfSetting.surfGroup.insert(std::make_pair(SurfaceTypeHVSTable, m_veboxDnHVSTables));
+
+    return MOS_STATUS_SUCCESS;
+}
+
 MOS_STATUS VpResourceManager::AssignSurface(VP_EXECUTE_CAPS caps, VEBOX_SURFACE_ID &surfaceId, SurfaceType surfaceType, VP_SURFACE *inputSurface, VP_SURFACE *outputSurface, VP_SURFACE *pastSurface, VP_SURFACE *futureSurface, VP_SURFACE_GROUP &surfGroup)
 {
     VP_FUNC_CALL();
@@ -1818,7 +2085,7 @@ MOS_STATUS VpResourceManager::AssignSurface(VP_EXECUTE_CAPS caps, VEBOX_SURFACE_
 
             if (!caps.bDN                               ||
                 nullptr == curDnOutputSurface           ||
-                // When FtrMediaTile64 is true, DN output surface will be tile64 when input is bayer format,
+                // When FtrTileY is false, DN output surface will be tile64 when input is bayer format,
                 // while pastSurface passed by OS maybe tile4, which is different from DN output surface.
                 // For such case, passSurface cannot be used, as vebox previous input surface and vebox
                 // DN output surface must share same setting. The derive pitch in vebox output surface
@@ -1866,7 +2133,7 @@ MOS_STATUS VpResourceManager::AssignSurface(VP_EXECUTE_CAPS caps, VEBOX_SURFACE_
 }
 
 MOS_STATUS VpResourceManager::AssignVeboxResource(VP_EXECUTE_CAPS& caps, VP_SURFACE *inputSurface, VP_SURFACE *outputSurface,
-    VP_SURFACE *pastSurface, VP_SURFACE *futureSurface, RESOURCE_ASSIGNMENT_HINT resHint, VP_SURFACE_SETTING &surfSetting)
+    VP_SURFACE *pastSurface, VP_SURFACE *futureSurface, RESOURCE_ASSIGNMENT_HINT resHint, VP_SURFACE_SETTING &surfSetting, SwFilterPipe& executedFilters)
 {
     VP_FUNC_CALL();
     VP_PUBLIC_CHK_NULL_RETURN(inputSurface);
@@ -1897,6 +2164,15 @@ MOS_STATUS VpResourceManager::AssignVeboxResource(VP_EXECUTE_CAPS& caps, VP_SURF
         auto it = m_veboxSurfaceConfigMap.find(cfg.value);
         if (m_veboxSurfaceConfigMap.end() == it)
         {
+            VP_PUBLIC_ASSERTMESSAGE("SurfaceConfig is invalid, cfg.value = %d", cfg.value);
+            VP_PUBLIC_ASSERTMESSAGE("b64Di = %d, sfcEnable = %d, sameSample = %d, outOfBound = %d, pastframeAvailable = %d, future frameAvaliable = %d, FirstDiFiels = %d",
+                cfg.b64DI,
+                cfg.sfcEnable,
+                cfg.sameSample,
+                cfg.outOfBound,
+                cfg.pastFrameAvailable,
+                cfg.futureFrameAvailable,
+                cfg.firstDiField);
             VP_PUBLIC_CHK_STATUS_RETURN(MOS_STATUS_INVALID_PARAMETER);
         }
         auto surfaces = it->second;
@@ -1949,14 +2225,28 @@ MOS_STATUS VpResourceManager::AssignVeboxResource(VP_EXECUTE_CAPS& caps, VP_SURF
     surfGroup.insert(std::make_pair(SurfaceTypeLaceAceRGBHistogram, m_veboxRgbHistogram));
 
     // Insert Vebox statistics surface
-    surfGroup.insert(std::make_pair(SurfaceTypeStatistics, m_veboxStatisticsSurface));
+    if (caps.b1stPassOfSfc2PassScaling)
+    {
+        surfGroup.insert(std::make_pair(SurfaceTypeStatistics, m_veboxStatisticsSurfacefor1stPassofSfc2Pass));
+    }
+    else
+    {
+        surfGroup.insert(std::make_pair(SurfaceTypeStatistics, m_veboxStatisticsSurface));
+    }
     surfSetting.dwVeboxPerBlockStatisticsHeight = m_dwVeboxPerBlockStatisticsHeight;
     surfSetting.dwVeboxPerBlockStatisticsWidth  = m_dwVeboxPerBlockStatisticsWidth;
 
     if (VeboxHdr3DlutNeeded(caps))
     {
         // Insert Vebox 3Dlut surface
-        surfGroup.insert(std::make_pair(SurfaceType3dLut, m_vebox3DLookUpTables));
+        surfGroup.insert(std::make_pair(SurfaceType3DLut, m_vebox3DLookUpTables));
+    }
+
+    if (resHint.isHVSTableNeeded)
+    {
+        VP_PUBLIC_CHK_NULL_RETURN(m_veboxDnHVSTables);
+        // Insert Vebox HVS DN surface
+        surfGroup.insert(std::make_pair(SurfaceTypeHVSTable, m_veboxDnHVSTables));
     }
 
     if (Vebox1DlutNeeded(caps))
@@ -2105,4 +2395,73 @@ bool VpResourceManager::VeboxSTMMNeeded(VP_EXECUTE_CAPS& caps, bool queryAssignm
     }
 }
 
+MOS_STATUS VpResourceManager::ReAllocateVeboxStatisticsSurface(VP_SURFACE *&statisticsSurface, VP_EXECUTE_CAPS &caps, VP_SURFACE *inputSurface, uint32_t dwWidth, uint32_t dwHeight)
+{
+    VP_FUNC_CALL();
+    
+    bool        bAllocated                  = false;
+    Mos_MemPool memTypeHistStat             = GetHistStatMemType(caps);
+    //Statistics surface can be not lockable, if secure mode is enabled
+    bool        isStatisticsBufNotLockable  = caps.bSecureVebox;
+    uint8_t     InitValue                   = 0;
+
+    // change the init value when null hw is enabled
+    if (m_osInterface.bNullHwIsEnabled)
+    {
+        InitValue = 0x80;
+    }
+
+    VP_PUBLIC_CHK_STATUS_RETURN(m_allocator.ReAllocateSurface(
+        statisticsSurface,
+        "m_veboxStatisticsSurface",
+        Format_Buffer,
+        MOS_GFXRES_BUFFER,
+        MOS_TILE_LINEAR,
+        dwWidth,
+        dwHeight,
+        false,
+        MOS_MMC_DISABLED,
+        bAllocated,
+        false,
+        IsDeferredResourceDestroyNeeded(),
+        MOS_HW_RESOURCE_USAGE_VP_INTERNAL_WRITE_FF,
+        MOS_TILE_UNSET_GMM,
+        memTypeHistStat,
+        isStatisticsBufNotLockable));
+
+    if (bAllocated)
+    {
+        if (caps.bSecureVebox)
+        {
+            VP_PUBLIC_CHK_STATUS_RETURN(FillLinearBufferWithEncZero(statisticsSurface, dwWidth, dwHeight));
+        }
+        else
+        {
+            // Initialize veboxStatisticsSurface Surface
+            VP_PUBLIC_CHK_STATUS_RETURN(m_allocator.OsFillResource(
+                &(statisticsSurface->osSurface->OsResource),
+                dwWidth * dwHeight,
+                InitValue));
+        }
+    }
+
+    m_dwVeboxPerBlockStatisticsWidth  = dwWidth;
+    m_dwVeboxPerBlockStatisticsHeight = MOS_ROUNDUP_DIVIDE(inputSurface->osSurface->dwHeight, 4);
+
+    return MOS_STATUS_SUCCESS;
+}
+
+MOS_STATUS VpResourceManager::AssignHdrResource(VP_EXECUTE_CAPS &caps, std::vector<VP_SURFACE *> &inputSurfaces, VP_SURFACE *outputSurface, RESOURCE_ASSIGNMENT_HINT resHint, VP_SURFACE_SETTING &surfSetting, SwFilterPipe &executedFilters)
+{
+    VP_FUNC_CALL();
+
+    if (m_hdrResourceManager == nullptr)
+    {
+        m_hdrResourceManager = MOS_New(VphdrResourceManager, m_allocator);
+    }
+
+    VP_PUBLIC_CHK_STATUS_RETURN(m_hdrResourceManager->AssignRenderResource(caps, inputSurfaces, outputSurface, resHint, surfSetting, executedFilters, m_osInterface, m_reporting, IsDeferredResourceDestroyNeeded()));
+
+    return MOS_STATUS_SUCCESS;
+}
 };
