@@ -28,6 +28,7 @@
 #include "vp_render_cmd_packet.h"
 #include "hw_filter.h"
 #include "sw_filter_pipe.h"
+#include "vp_hal_ddi_utils.h"
 #include <vector>
 
 namespace vp {
@@ -435,7 +436,7 @@ MOS_STATUS VpFcFilter::CalculateScalingParams(VP_FC_LAYER *layer, VP_FC_LAYER *t
 
     if (VPHAL_SCALING_NEAREST == layer->scalingMode && (isChromaUpSamplingNeeded || isChromaDownSamplingNeeded))
     {
-        VP_PUBLIC_ASSERTMESSAGE("Scaling Info: Nearest scaling with isChromaUpSamplingNeeded (%d) and isChromaDownSamplingNeeded (%d)",
+        VP_PUBLIC_NORMALMESSAGE("Scaling Info: Nearest scaling with isChromaUpSamplingNeeded (%d) and isChromaDownSamplingNeeded (%d)",
             isChromaUpSamplingNeeded, isChromaDownSamplingNeeded);
     }
 
@@ -681,27 +682,37 @@ MOS_STATUS PolicyFcFeatureHandler::UpdateFeaturePipe(VP_EXECUTE_CAPS caps, SwFil
 
     FeatureType type = feature.GetFeatureType();
 
-    if (FeatureTypeLumakeyOnRender      == type ||
-        FeatureTypeBlendingOnRender     == type ||
-        FeatureTypeAlphaOnRender        == type ||
-        FeatureTypeCscOnRender          == type ||
-        FeatureTypeScalingOnRender      == type ||
-        FeatureTypeRotMirOnRender       == type ||
-        FeatureTypeDiOnRender           == type ||
-        FeatureTypeProcampOnRender      == type)
+    if (caps.bRenderHdr)
     {
+        // HDR Kernel
         return PolicyFeatureHandler::UpdateFeaturePipe(caps, feature, featurePipe, executePipe, isInputPipe, index);
     }
-    else if(FeatureTypeColorFillOnRender == type)
-    {
-        // Only apply color fill on 1st pass.
-        featurePipe.RemoveSwFilter(&feature);
-        executePipe.AddSwFilterUnordered(&feature, isInputPipe, index);
-    }
     else
     {
-        VP_PUBLIC_CHK_STATUS_RETURN(MOS_STATUS_INVALID_PARAMETER);
+        // FC
+        if (FeatureTypeLumakeyOnRender      == type ||
+            FeatureTypeBlendingOnRender     == type ||
+            FeatureTypeAlphaOnRender        == type ||
+            FeatureTypeCscOnRender          == type ||
+            FeatureTypeScalingOnRender      == type ||
+            FeatureTypeRotMirOnRender       == type ||
+            FeatureTypeDiOnRender           == type ||
+            FeatureTypeProcampOnRender      == type)
+        {
+            return PolicyFeatureHandler::UpdateFeaturePipe(caps, feature, featurePipe, executePipe, isInputPipe, index);
+        }
+        else if(FeatureTypeColorFillOnRender == type)
+        {
+            // Only apply color fill on 1st pass.
+            featurePipe.RemoveSwFilter(&feature);
+            executePipe.AddSwFilterUnordered(&feature, isInputPipe, index);
+        }
+        else
+        {
+            VP_PUBLIC_CHK_STATUS_RETURN(MOS_STATUS_INVALID_PARAMETER);
+        }
     }
+
     return MOS_STATUS_SUCCESS;
 }
 
@@ -714,7 +725,7 @@ MOS_STATUS PolicyFcFeatureHandler::UpdateUnusedFeature(VP_EXECUTE_CAPS caps, SwF
     if (0 == caps.bOutputPipeFeatureInuse &&
         !feature.GetFilterEngineCaps().bEnabled &&
         (feature.GetFilterEngineCaps().forceEnableForSfc ||
-         feature.GetFilterEngineCaps().forceEnableForRender))
+         feature.GetFilterEngineCaps().forceEnableForFc))
     {
         // To avoid filter being destroyed in Policy::UpdateFeaturePipe.
         feature.GetFilterEngineCaps().usedForNextPass = 1;
@@ -806,8 +817,8 @@ static MOS_STATUS IsChromaSamplingNeeded(bool &isChromaUpSamplingNeeded, bool &i
                                 VPHAL_SURFACE_TYPE surfType, int layerIndex,
                                 MOS_FORMAT inputFormat, MOS_FORMAT outputFormat)
 {
-    VPHAL_COLORPACK srcColorPack = VpHal_GetSurfaceColorPack(inputFormat);
-    VPHAL_COLORPACK dstColorPack = VpHal_GetSurfaceColorPack(outputFormat);
+    VPHAL_COLORPACK srcColorPack = VpHalDDIUtils::GetSurfaceColorPack(inputFormat);
+    VPHAL_COLORPACK dstColorPack = VpHalDDIUtils::GetSurfaceColorPack(outputFormat);
 
     if (SURF_IN_PRIMARY == surfType                         &&
         // when 3D sampler been used, PL2 chromasitting kernel does not support sub-layer chromasitting
