@@ -25,10 +25,11 @@
 //! \details  render packet provide the structures and generate the cmd buffer which mediapipline will used.
 //!
 #include "vp_platform_interface.h"
-#include "cm_visa.h"
+#include "vp_visa.h"
+#include "vp_user_setting.h"
 
 using namespace vp;
-
+extern const Kdll_RuleEntry g_KdllRuleTable_Next[];
 const std::string VpRenderKernel::s_kernelNameNonAdvKernels = "vpFcKernels";
 
 VpPlatformInterface::VpPlatformInterface(PMOS_INTERFACE pOsInterface)
@@ -38,7 +39,7 @@ VpPlatformInterface::VpPlatformInterface(PMOS_INTERFACE pOsInterface)
     {
         m_userSettingPtr = m_pOsInterface->pfnGetUserSettingInstance(m_pOsInterface);
     }
-    VpUtils::DeclareUserSettings(m_userSettingPtr);
+    VpUserSetting::InitVpUserSetting(m_userSettingPtr);
 }
 
 MOS_STATUS VpRenderKernel::InitVPKernel(
@@ -234,9 +235,105 @@ MOS_STATUS VpRenderKernel::AddKernelArg(KRN_ARG &kernelArg)
     return MOS_STATUS_SUCCESS;
 }
 
+void VpPlatformInterface::AddVpIsaKernelEntryToList(
+    const uint32_t *kernelBin,
+    uint32_t        kernelBinSize,
+    std::string     postfix,
+    DelayLoadedKernelType delayKernelType)
+{
+    VP_FUNC_CALL();
+
+    VP_KERNEL_BINARY_ENTRY tmpEntry = {};
+    tmpEntry.kernelBin     = kernelBin;
+    tmpEntry.kernelBinSize = kernelBinSize;
+    tmpEntry.postfix       = postfix;
+    tmpEntry.kernelType    = delayKernelType;
+
+    if (delayKernelType == KernelNone)
+    {
+        m_vpIsaKernelBinaryList.push_back(tmpEntry);
+    }
+    else
+    {
+        m_vpDelayLoadedBinaryList.push_back(tmpEntry);
+        m_vpDelayLoadedFeatureSet.insert(std::make_pair(delayKernelType, false));
+    }
+}
+
+void VpPlatformInterface::AddVpL0KernelEntryToList(
+    const uint32_t *kernelBin,
+    uint32_t        kernelBinSize,
+    std::string     kernelName)
+{
+    VP_FUNC_CALL();
+
+    VP_KERNEL_BINARY_ENTRY tmpEntry = {};
+    tmpEntry.kernelBin     = kernelBin;
+    tmpEntry.kernelBinSize = kernelBinSize;
+
+    m_vpL0KernelBinaryList.insert(std::make_pair(kernelName, tmpEntry));
+}
+
+void       KernelDll_ModifyFunctionPointers_Next(Kdll_State *pState);
+
+MOS_STATUS VpPlatformInterface::InitVpRenderHwCaps()
+{
+    VP_FUNC_CALL();
+    VP_RENDER_CHK_NULL_RETURN(m_vpKernelBinary.kernelBin);
+    VP_RENDER_CHK_NULL_RETURN(m_vpKernelBinary.fcPatchKernelBin);
+    // Only Lpm Plus will use this base function
+    m_modifyKdllFunctionPointers = KernelDll_ModifyFunctionPointers_Next;
+#if defined(ENABLE_KERNELS)
+    InitVPFCKernels(
+        g_KdllRuleTable_Next,
+        m_vpKernelBinary.kernelBin,
+        m_vpKernelBinary.kernelBinSize,
+        m_vpKernelBinary.fcPatchKernelBin,
+        m_vpKernelBinary.fcPatchKernelBinSize,
+        m_modifyKdllFunctionPointers);
+#endif
+
+    if (!m_vpIsaKernelBinaryList.empty())
+    {
+        // Init CM kernel form VP ISA Kernel Binary List
+        for (auto &curKernelEntry : m_vpIsaKernelBinaryList)
+        {
+            VP_PUBLIC_CHK_STATUS_RETURN(InitVpCmKernels(curKernelEntry.kernelBin, curKernelEntry.kernelBinSize, curKernelEntry.postfix));
+        }
+    }
+
+    if (!m_vpL0KernelBinaryList.empty())
+    {
+        // Init L0 kernel form VP L0 Kernel Binary List
+        for (auto &curKernelEntry : m_vpL0KernelBinaryList)
+        {
+            VP_PUBLIC_CHK_STATUS_RETURN(InitVpL0Kernels(curKernelEntry.first, curKernelEntry.second));
+        }
+    }
+    return MOS_STATUS_SUCCESS;
+}
+
+MOS_STATUS VpPlatformInterface::InitVpL0Kernels(
+    std::string            kernelName,
+    VP_KERNEL_BINARY_ENTRY kernelBinaryEntry)
+{
+    VP_FUNC_CALL();
+
+    VpRenderKernel vpKernel;
+
+    vpKernel.SetKernelBinPointer((void *)kernelBinaryEntry.kernelBin);
+    vpKernel.SetKernelName(kernelName);
+    vpKernel.SetKernelBinOffset(0x0);
+    vpKernel.SetKernelBinSize(kernelBinaryEntry.kernelBinSize);
+    m_kernelPool.insert(std::make_pair(vpKernel.GetKernelName(), vpKernel));
+
+    return MOS_STATUS_SUCCESS;
+}
+
 MOS_STATUS VpPlatformInterface::InitVpCmKernels(
     const uint32_t *cisaCode,
-    uint32_t        cisaCodeSize)
+    uint32_t        cisaCodeSize,
+    std::string     postfix)
 {
     VP_FUNC_CALL();
     VP_RENDER_CHK_NULL_RETURN(cisaCode);
@@ -287,14 +384,19 @@ MOS_STATUS VpPlatformInterface::InitVpCmKernels(
         }
 
         std::string kernelName(kernel->getName(), kernel->getNameLen());
+        std::string fullKernelName = kernelName;
+        if (!postfix.empty())
+        {
+            fullKernelName += ('_' + postfix);
+        }
 
-        if (m_kernelPool.end() != m_kernelPool.find(kernelName))
+        if (m_kernelPool.end() != m_kernelPool.find(fullKernelName))
         {
             continue;
         }
 
         VpRenderKernel vpKernel;
-        vpKernel.SetKernelName(kernelName);
+        vpKernel.SetKernelName(fullKernelName);
         vpKernel.SetKernelBinPointer((void *)cisaCode);
 
         uint8_t          numGenBinaries = kernel->getNumGenBinaries();
@@ -380,6 +482,11 @@ VpPlatformInterface::~VpPlatformInterface()
     {
         kernel.second.Destroy();
     }
+
+    if (!m_vpDelayLoadedBinaryList.empty())
+    {
+        m_vpDelayLoadedBinaryList.clear();
+    }
 }
 
 MOS_STATUS VpPlatformInterface::GetKernelParam(VpKernelID kernlId, RENDERHAL_KERNEL_PARAM &param)
@@ -390,7 +497,7 @@ MOS_STATUS VpPlatformInterface::GetKernelParam(VpKernelID kernlId, RENDERHAL_KER
     return MOS_STATUS_SUCCESS;
 }
 
-void VpPlatformInterface::SetVpKernelBinary(
+void VpPlatformInterface::SetVpFCKernelBinary(
                 const uint32_t   *kernelBin,
                 uint32_t         kernelBinSize,
                 const uint32_t   *fcPatchKernelBin,
@@ -404,18 +511,28 @@ void VpPlatformInterface::SetVpKernelBinary(
     m_vpKernelBinary.fcPatchKernelBinSize = fcPatchKernelBinSize;
 }
 
-void VpPlatformInterface::SetVpISAKernelBinary(
-                const uint32_t   *isa3DLUTKernelBin,
-                uint32_t         isa3DLUTKernelSize,
-                const uint32_t   *isaHVSDenoiseKernelBin,
-                uint32_t         isaHVSDenoiseKernelSize)
+MOS_STATUS VpPlatformInterface::InitializeDelayedKernels(DelayLoadedKernelType type)
 {
     VP_FUNC_CALL();
-    
-    m_vpKernelBinary.isa3DLUTKernelBin       = isa3DLUTKernelBin;
-    m_vpKernelBinary.isa3DLUTKernelSize      = isa3DLUTKernelSize;
-    m_vpKernelBinary.isaHVSDenoiseKernelBin  = isaHVSDenoiseKernelBin;
-    m_vpKernelBinary.isaHVSDenoiseKernelSize = isaHVSDenoiseKernelSize;
+    auto feature = m_vpDelayLoadedFeatureSet.find(type);
+    if (feature != m_vpDelayLoadedFeatureSet.end() && feature->second == false && !m_vpDelayLoadedBinaryList.empty())
+    {
+        // Init CM kernel form VP ISA Kernel Binary List
+        for (auto it = m_vpDelayLoadedBinaryList.begin(); it != m_vpDelayLoadedBinaryList.end();)
+        {
+            if (it->kernelType == type)
+            {
+                VP_PUBLIC_CHK_STATUS_RETURN(InitVpCmKernels(it->kernelBin, it->kernelBinSize, it->postfix));
+                m_vpDelayLoadedBinaryList.erase(it);
+            }
+            else
+            {
+                ++it;
+            }
+        }
+        feature->second = true;
+    }
+    return MOS_STATUS_SUCCESS;
 }
 
 //only for get kernel binary in legacy path not being used in APO path.
@@ -465,12 +582,12 @@ MOS_STATUS VpPlatformInterface::GetVeboxHeapInfo(
     return eStatus;
 }
 
-bool VpPlatformInterface::IsVeboxScalabilitywith4K(
+bool VpPlatformInterface::IsVeboxScalabilityWith4KNotSupported(
     VP_MHWINTERFACE          vpMhwInterface)
 {
-    if(m_veboxItf)
+    if (m_veboxItf && !(m_veboxItf->IsVeboxScalabilitywith4K()))
     {
-        return m_veboxItf->IsVeboxScalabilitywith4K();
+        return true;
     }
     else
     {
